@@ -15,27 +15,27 @@
  * 02110-1301, USA.
  */
 
-use libc;
-use std;
-use json::*;
 use super::template::TemplateTransaction;
+use crate::jsonbuilder::{JsonBuilder, JsonError};
+use std;
 
-fn log_template(tx: &TemplateTransaction) -> Option<Json> {
-    let js = Json::object();
+fn log_template(tx: &TemplateTransaction, js: &mut JsonBuilder) -> Result<(), JsonError> {
+    js.open_object("template")?;
     if let Some(ref request) = tx.request {
-        js.set_string("request", request);
+        js.set_string("request", request)?;
     }
     if let Some(ref response) = tx.response {
-        js.set_string("response", response);
+        js.set_string("response", response)?;
     }
-    return Some(js);
+    js.close()?;
+    Ok(())
 }
 
 #[no_mangle]
-pub extern "C" fn rs_template_logger_log(tx: *mut libc::c_void) -> *mut JsonT {
+pub unsafe extern "C" fn SCTemplateLoggerLog(
+    tx: *const std::os::raw::c_void, js: *mut std::os::raw::c_void,
+) -> bool {
     let tx = cast_pointer!(tx, TemplateTransaction);
-    match log_template(tx) {
-        Some(js) => js.unwrap(),
-        None => std::ptr::null_mut(),
-    }
+    let js = cast_pointer!(js, JsonBuilder);
+    log_template(tx, js).is_ok()
 }
