@@ -22,8 +22,8 @@
  * \author Duarte Silva <duarte.silva@serializing.me>
  */
 
-#ifndef __APP_LAYER_HTP_XFF_H__
-#define __APP_LAYER_HTP_XFF_H__
+#ifndef SURICATA_APP_LAYER_HTP_XFF_H
+#define SURICATA_APP_LAYER_HTP_XFF_H
 
 /** XFF is disabled */
 #define XFF_DISABLED 1
@@ -43,12 +43,12 @@ typedef struct HttpXFFCfg_ {
     const char *header; /**< XFF header name */
 } HttpXFFCfg;
 
-void HttpXFFGetCfg(ConfNode *conf, HttpXFFCfg *result);
+void HttpXFFGetCfg(SCConfNode *conf, HttpXFFCfg *result);
 
-int HttpXFFGetIPFromTx(const Packet *p, uint64_t tx_id, HttpXFFCfg *xff_cfg, char *dstbuf, int dstbuflen);
+int HttpXFFGetIPFromTx(const Flow *f, uint64_t tx_id, HttpXFFCfg *xff_cfg, char *dstbuf, int dstbuflen);
 
-int HttpXFFGetIP(const Packet *p, HttpXFFCfg *xff_cfg, char *dstbuf, int dstbuflen);
+int HttpXFFGetIP(const Flow *f, HttpXFFCfg *xff_cfg, char *dstbuf, int dstbuflen);
 
 void HTPXFFParserRegisterTests(void);
 
-#endif /* __APP_LAYER_HTP_XFF_H__ */
+#endif /* SURICATA_APP_LAYER_HTP_XFF_H */
