@@ -22,8 +22,10 @@
  * \author Victor Julien <victor@inliniac.net>
  */
 
-#ifndef __COUNTERS_H__
-#define __COUNTERS_H__
+#ifndef SURICATA_COUNTERS_H
+#define SURICATA_COUNTERS_H
+
+#include "threads.h"
 
 /* forward declaration of the ThreadVars structure */
 struct ThreadVars_;
@@ -41,7 +43,7 @@ typedef struct StatsCounter_ {
     uint16_t gid;
 
     /* counter value(s): copies from the 'private' counter */
-    uint64_t value;     /**< sum of updates/increments, or 'set' value */
+    int64_t value;      /**< sum of updates/increments, or 'set' value */
     uint64_t updates;   /**< number of updates (for avg) */
 
     /* when using type STATS_TYPE_Q_FUNC this function is called once
@@ -50,6 +52,7 @@ typedef struct StatsCounter_ {
 
     /* name of the counter */
     const char *name;
+    const char *short_name;
 
     /* the next perfcounter for this tv's tm instance */
     struct StatsCounter_ *next;
@@ -60,7 +63,7 @@ typedef struct StatsCounter_ {
  */
 typedef struct StatsPublicThreadContext_ {
     /* flag set by the wakeup thread, to inform the client threads to sync */
-    uint32_t perf_flag;
+    SC_ATOMIC_DECLARE(bool, sync_now);
 
     /* pointer to the head of a list of counters assigned under this context */
     StatsCounter *head;
@@ -84,7 +87,7 @@ typedef struct StatsLocalCounter_ {
     uint16_t id;
 
     /* total value of the adds/increments, or exact value in case of 'set' */
-    uint64_t value;
+    int64_t value;
 
     /* no of times the local counter has been updated */
     uint64_t updates;
@@ -105,11 +108,13 @@ typedef struct StatsPrivateThreadContext_ {
 
 /* the initialization functions */
 void StatsInit(void);
-void StatsSetupPostConfig(void);
+void StatsSetupPostConfigPreOutput(void);
+void StatsSetupPostConfigPostOutput(void);
 void StatsSpawnThreads(void);
 void StatsRegisterTests(void);
+bool StatsEnabled(void);
 
-/* functions used to free the resources alloted by the Stats API */
+/* functions used to free the resources allotted by the Stats API */
 void StatsReleaseResources(void);
 
 /* counter registration functions */
@@ -122,6 +127,7 @@ uint16_t StatsRegisterGlobalCounter(const char *cname, uint64_t (*Func)(void));
 void StatsAddUI64(struct ThreadVars_ *, uint16_t, uint64_t);
 void StatsSetUI64(struct ThreadVars_ *, uint16_t, uint64_t);
 void StatsIncr(struct ThreadVars_ *, uint16_t);
+void StatsDecr(struct ThreadVars_ *, uint16_t);
 
 /* utility functions */
 int StatsUpdateCounterArray(StatsPrivateThreadContext *, StatsPublicThreadContext *);
@@ -129,21 +135,12 @@ uint64_t StatsGetLocalCounterValue(struct ThreadVars_ *, uint16_t);
 int StatsSetupPrivate(struct ThreadVars_ *);
 void StatsThreadCleanup(struct ThreadVars_ *);
 
-#define StatsSyncCounters(tv) \
-    StatsUpdateCounterArray(&(tv)->perf_private_ctx, &(tv)->perf_public_ctx);  \
-
-#define StatsSyncCountersIfSignalled(tv)                                       \
-    do {                                                                        \
-        if ((tv)->perf_public_ctx.perf_flag == 1) {                             \
-            StatsUpdateCounterArray(&(tv)->perf_private_ctx,                   \
-                                     &(tv)->perf_public_ctx);                   \
-        }                                                                       \
-    } while (0)
+void StatsSyncCounters(struct ThreadVars_ *tv);
+void StatsSyncCountersIfSignalled(struct ThreadVars_ *tv);
 
 #ifdef BUILD_UNIX_SOCKET
 TmEcode StatsOutputCounterSocket(json_t *cmd,
                                  json_t *answer, void *data);
 #endif
 
-#endif /* __COUNTERS_H__ */
-
+#endif /* SURICATA_COUNTERS_H */
