@@ -23,15 +23,13 @@
  * Generic Route Encapsulation (GRE) from RFC 1701.
  */
 
-#ifndef __DECODE_GRE_H__
-#define __DECODE_GRE_H__
+#ifndef SURICATA_DECODE_GRE_H
+#define SURICATA_DECODE_GRE_H
 
 #ifndef IPPROTO_GRE
 #define IPPROTO_GRE 47
 #endif
 
-#include "decode.h"
-#include "threadvars.h"
 
 typedef struct GREHdr_
 {
@@ -41,6 +39,13 @@ typedef struct GREHdr_
 
 } __attribute__((__packed__)) GREHdr;
 
+/* Enhanced GRE header - https://tools.ietf.org/html/rfc2637#section-4.1 */
+typedef struct GREPPtPHdr_ {
+    GREHdr greh;             /** base GRE packet header */
+    uint16_t payload_length; /** PPP payload length */
+    uint16_t call_id;        /** PPP peer id */
+} __attribute__((__packed__)) GREPPtPHd;
+
 /* Generic Routing Encapsulation Source Route Entries (SREs).
  * The header is followed by a variable amount of Routing Information.
  */
@@ -70,7 +75,7 @@ typedef struct GRESreHdr_
 #define GRE_FLAG_ISSET_RECUR(r)     (r->flags & 0x07)
 #define GRE_GET_VERSION(r)   (r->version & 0x07)
 #define GRE_GET_FLAGS(r)     (r->version & 0xF8)
-#define GRE_GET_PROTO(r)     ntohs(r->ether_type)
+#define GRE_GET_PROTO(r)     SCNtohs(r->ether_type)
 
 #define GREV1_HDR_LEN           8
 #define GREV1_ACK_LEN           4
@@ -79,5 +84,4 @@ typedef struct GRESreHdr_
 
 void DecodeGRERegisterTests(void);
 
-#endif /* __DECODE_GRE_H__ */
-
+#endif /* SURICATA_DECODE_GRE_H */
