@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2010 Open Information Security Foundation
+/* Copyright (C) 2007-2022 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -21,20 +21,9 @@
  * \author Anoop Saldanha <anoopsaldanha@gmail.com>
  */
 
-#ifndef __DETECT_APP_LAYER_EVENT_H__
-#define __DETECT_APP_LAYER_EVENT_H__
+#ifndef SURICATA_DETECT_APP_LAYER_EVENT_H
+#define SURICATA_DETECT_APP_LAYER_EVENT_H
 
-typedef struct DetectAppLayerEventData_ {
-    AppProto alproto;
-    int event_id;
-
-    /* it's used to check if there are event set into the detect engine */
-    bool needs_detctx;
-
-    char *arg;
-} DetectAppLayerEventData;
-
-int DetectAppLayerEventPrepare(Signature *s);
 void DetectAppLayerEventRegister(void);
 
-#endif /* __DETECT_APP_LAYER_EVENT_H__ */
+#endif /* SURICATA_DETECT_APP_LAYER_EVENT_H */
