@@ -15,14 +15,14 @@
  * 02110-1301, USA.
  */
 
-#ifndef __DETECT_DNP3_H__
-#define __DETECT_DNP3_H__
+#ifndef SURICATA_DETECT_DNP3_H
+#define SURICATA_DETECT_DNP3_H
 
 /**
  * Struct for mapping symbolic names to values.
  */
 typedef struct DNP3Mapping_ {
-    char     *name;
+    const char     *name;
     uint16_t  value;
 } DNP3Mapping;
 
@@ -31,4 +31,4 @@ extern DNP3Mapping DNP3IndicatorsMap[];
 
 void DetectDNP3Register(void);
 
-#endif /* __DETECT_DNP3_H__ */
+#endif /* SURICATA_DETECT_DNP3_H */
