@@ -15,14 +15,16 @@
  * 02110-1301, USA.
  */
 
-#ifndef __DETECT_ENGINE_PREFILTER_COMMON_H__
-#define __DETECT_ENGINE_PREFILTER_COMMON_H__
+#ifndef SURICATA_DETECT_ENGINE_PREFILTER_COMMON_H
+#define SURICATA_DETECT_ENGINE_PREFILTER_COMMON_H
+
+#include "rust.h"
 
 typedef union {
-    uint8_t u8[8];
-    uint16_t u16[4];
-    uint32_t u32[2];
-    uint64_t u64;
+    uint8_t u8[16];
+    uint16_t u16[8];
+    uint32_t u32[4];
+    uint64_t u64[2];
 } PrefilterPacketHeaderValue;
 
 #define PREFILTER_EXTRA_MATCH_UNUSED  0
@@ -51,26 +53,22 @@ typedef struct PrefilterPacketU8HashCtx_ {
     SigsArray *array[256];
 } PrefilterPacketU8HashCtx;
 
-#define PREFILTER_U8HASH_MODE_EQ    0
-#define PREFILTER_U8HASH_MODE_LT    1
-#define PREFILTER_U8HASH_MODE_GT    2
-#define PREFILTER_U8HASH_MODE_RA    3
+#define PREFILTER_U8HASH_MODE_EQ DetectUintModeEqual
+#define PREFILTER_U8HASH_MODE_LT DetectUintModeLt
+#define PREFILTER_U8HASH_MODE_GT DetectUintModeGt
+#define PREFILTER_U8HASH_MODE_RA DetectUintModeRange
 
-int PrefilterSetupPacketHeader(DetectEngineCtx *de_ctx,
-        SigGroupHead *sgh, int sm_type,
-        void (*Set)(PrefilterPacketHeaderValue *v, void *),
-        _Bool (*Compare)(PrefilterPacketHeaderValue v, void *),
-        void (*Match)(DetectEngineThreadCtx *det_ctx,
-            Packet *p, const void *pectx));
+int PrefilterSetupPacketHeader(DetectEngineCtx *de_ctx, SigGroupHead *sgh, int sm_type,
+        SignatureMask mask, void (*Set)(PrefilterPacketHeaderValue *v, void *),
+        bool (*Compare)(PrefilterPacketHeaderValue v, void *),
+        void (*Match)(DetectEngineThreadCtx *det_ctx, Packet *p, const void *pectx));
 
-int PrefilterSetupPacketHeaderU8Hash(DetectEngineCtx *de_ctx,
-        SigGroupHead *sgh, int sm_type,
-        void (*Set)(PrefilterPacketHeaderValue *v, void *),
-        _Bool (*Compare)(PrefilterPacketHeaderValue v, void *),
-        void (*Match)(DetectEngineThreadCtx *det_ctx,
-            Packet *p, const void *pectx));
+int PrefilterSetupPacketHeaderU8Hash(DetectEngineCtx *de_ctx, SigGroupHead *sgh, int sm_type,
+        SignatureMask mask, void (*Set)(PrefilterPacketHeaderValue *v, void *),
+        bool (*Compare)(PrefilterPacketHeaderValue v, void *),
+        void (*Match)(DetectEngineThreadCtx *det_ctx, Packet *p, const void *pectx));
 
-static inline _Bool
+static inline bool
 PrefilterPacketHeaderExtraMatch(const PrefilterPacketHeaderCtx *ctx,
                                 const Packet *p)
 {
@@ -79,19 +77,30 @@ PrefilterPacketHeaderExtraMatch(const PrefilterPacketHeaderCtx *ctx,
         case PREFILTER_EXTRA_MATCH_UNUSED:
             break;
         case PREFILTER_EXTRA_MATCH_ALPROTO:
-            if (p->flow == NULL || p->flow->alproto != ctx->value)
-                return FALSE;
+            if (p->flow == NULL || !AppProtoEquals(ctx->value, p->flow->alproto))
+                return false;
             break;
         case PREFILTER_EXTRA_MATCH_SRCPORT:
             if (p->sp != ctx->value)
-                return FALSE;
+                return false;
             break;
         case PREFILTER_EXTRA_MATCH_DSTPORT:
             if (p->dp != ctx->value)
-                return FALSE;
+                return false;
             break;
     }
-    return TRUE;
+    return true;
 }
 
-#endif /* __DETECT_ENGINE_PREFILTER_COMMON_H__ */
+static inline bool PrefilterIsPrefilterableById(const Signature *s, enum DetectKeywordId kid)
+{
+    const SigMatch *sm;
+    for (sm = s->init_data->smlists[DETECT_SM_LIST_MATCH] ; sm != NULL; sm = sm->next) {
+        if (sm->type == kid) {
+            return true;
+        }
+    }
+    return false;
+}
+
+#endif /* SURICATA_DETECT_ENGINE_PREFILTER_COMMON_H */
