@@ -24,6 +24,12 @@
 #ifndef _DETECT_ENGINE_PROFILE_H
 #define	_DETECT_ENGINE_PROFILE_H
 
-void RulesDumpMatchArray(const DetectEngineThreadCtx *det_ctx, const Packet *p);
+#include "detect.h"
+
+void RulesDumpTxMatchArray(const DetectEngineThreadCtx *det_ctx, const SigGroupHead *sgh,
+        const Packet *p, const uint64_t tx_id, const uint32_t rule_cnt,
+        const uint32_t pkt_prefilter_cnt);
+void RulesDumpMatchArray(const DetectEngineThreadCtx *det_ctx,
+        const SigGroupHead *sgh, const Packet *p);
 
 #endif	/* _DETECT_ENGINE_PROFILE_H */
