@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2011 Open Information Security Foundation
+/* Copyright (C) 2007-2022 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -21,27 +21,13 @@
  * \author Victor Julien <victor@inliniac.net>
  */
 
-#ifndef __DETECT_FILEMAGIC_H__
-#define __DETECT_FILEMAGIC_H__
+#ifndef SURICATA_DETECT_FILEMAGIC_H
+#define SURICATA_DETECT_FILEMAGIC_H
 
 #ifdef HAVE_MAGIC
-#include "util-spm-bm.h"
-
-typedef struct DetectFilemagicThreadData {
-    magic_t ctx;
-} DetectFilemagicThreadData;
-
-typedef struct DetectFilemagicData {
-    int thread_ctx_id;
-    uint8_t *name; /** name of the file to match */
-    BmCtx *bm_ctx; /** BM context */
-    uint16_t len; /** name length */
-    uint32_t flags;
-} DetectFilemagicData;
-
 /* prototypes */
-int FilemagicGlobalLookup(File *file);
+int FilemagicThreadLookup(magic_t *ctx, File *file);
 #endif
 void DetectFilemagicRegister (void);
 
-#endif /* __DETECT_FILEMAGIC_H__ */
+#endif /* SURICATA_DETECT_FILEMAGIC_H */
