@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2012 Open Information Security Foundation
+/* Copyright (C) 2007-2021 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -25,7 +25,6 @@
 
 #include "suricata-common.h"
 #include "threads.h"
-#include "debug.h"
 #include "decode.h"
 
 #include "detect.h"
@@ -35,6 +34,8 @@
 #include "detect-engine-mpm.h"
 #include "detect-engine-state.h"
 
+#include "feature.h"
+
 #include "flow.h"
 #include "flow-var.h"
 #include "flow-util.h"
@@ -52,19 +53,24 @@
 
 #include "detect-filestore.h"
 
+#include "util-validate.h"
+
 /**
  * \brief Regex for parsing our flow options
  */
 #define PARSE_REGEX  "^\\s*([A-z_]+)\\s*(?:,\\s*([A-z_]+))?\\s*(?:,\\s*([A-z_]+))?\\s*$"
 
-static pcre *parse_regex;
-static pcre_extra *parse_regex_study;
+static DetectParseRegex parse_regex;
 
-static int DetectFilestoreMatch (ThreadVars *, DetectEngineThreadCtx *,
+static int DetectFilestoreMatch (DetectEngineThreadCtx *,
         Flow *, uint8_t, File *, const Signature *, const SigMatchCtx *);
+static int DetectFilestorePostMatch(DetectEngineThreadCtx *det_ctx,
+        Packet *p, const Signature *s, const SigMatchCtx *ctx);
 static int DetectFilestoreSetup (DetectEngineCtx *, Signature *, const char *);
-static void DetectFilestoreFree(void *);
+static void DetectFilestoreFree(DetectEngineCtx *, void *);
+#ifdef UNITTESTS
 static void DetectFilestoreRegisterTests(void);
+#endif
 static int g_file_match_list_id = 0;
 
 /**
@@ -74,14 +80,20 @@ void DetectFilestoreRegister(void)
 {
     sigmatch_table[DETECT_FILESTORE].name = "filestore";
     sigmatch_table[DETECT_FILESTORE].desc = "stores files to disk if the rule matched";
-    sigmatch_table[DETECT_FILESTORE].url = DOC_URL DOC_VERSION "/rules/file-keywords.html#filestore";
+    sigmatch_table[DETECT_FILESTORE].url = "/rules/file-keywords.html#filestore";
     sigmatch_table[DETECT_FILESTORE].FileMatch = DetectFilestoreMatch;
     sigmatch_table[DETECT_FILESTORE].Setup = DetectFilestoreSetup;
     sigmatch_table[DETECT_FILESTORE].Free  = DetectFilestoreFree;
+#ifdef UNITTESTS
     sigmatch_table[DETECT_FILESTORE].RegisterTests = DetectFilestoreRegisterTests;
+#endif
     sigmatch_table[DETECT_FILESTORE].flags = SIGMATCH_OPTIONAL_OPT;
 
-    DetectSetupParseRegexes(PARSE_REGEX, &parse_regex, &parse_regex_study);
+    sigmatch_table[DETECT_FILESTORE_POSTMATCH].name = "__filestore__postmatch__";
+    sigmatch_table[DETECT_FILESTORE_POSTMATCH].Match = DetectFilestorePostMatch;
+    sigmatch_table[DETECT_FILESTORE_POSTMATCH].Free  = DetectFilestoreFree;
+
+    DetectSetupParseRegexes(PARSE_REGEX, &parse_regex);
 
     g_file_match_list_id = DetectBufferTypeRegister("files");
 }
@@ -106,7 +118,8 @@ static int FilestorePostMatchWithOptions(Packet *p, Flow *f, const DetectFilesto
     switch (filestore->direction) {
         case FILESTORE_DIR_DEFAULT:
             rule_dir = 1;
-            break;
+            // will use both sides if scope is not default
+            // fallthrough
         case FILESTORE_DIR_BOTH:
             toserver_dir = 1;
             toclient_dir = 1;
@@ -140,30 +153,32 @@ static int FilestorePostMatchWithOptions(Packet *p, Flow *f, const DetectFilesto
     if (this_file)  {
         FileStoreFileById(fc, file_id);
     } else if (this_tx) {
-        /* flag tx all files will be stored */
-        if (f->alproto == ALPROTO_HTTP && f->alstate != NULL) {
-            HtpState *htp_state = f->alstate;
-            if (toserver_dir) {
-                htp_state->flags |= HTP_FLAG_STORE_FILES_TX_TS;
-                FileStoreAllFilesForTx(htp_state->files_ts, tx_id);
-            }
+        /* set in AppLayerTxData. Parsers and logger will propagate it to the
+         * individual files, both new and current. */
+        void *txv = AppLayerParserGetTx(f->proto, f->alproto, f->alstate, tx_id);
+        DEBUG_VALIDATE_BUG_ON(txv == NULL);
+        if (txv != NULL) {
+            AppLayerTxData *txd = AppLayerParserGetTxData(f->proto, f->alproto, txv);
             if (toclient_dir) {
-                htp_state->flags |= HTP_FLAG_STORE_FILES_TX_TC;
-                FileStoreAllFilesForTx(htp_state->files_tc, tx_id);
+                txd->file_flags |= FLOWFILE_STORE_TC;
             }
-            htp_state->store_tx_id = tx_id;
+                if (toserver_dir) {
+                    txd->file_flags |= FLOWFILE_STORE_TS;
+                }
         }
     } else if (this_flow) {
-        /* flag flow all files will be stored */
-        if (f->alproto == ALPROTO_HTTP && f->alstate != NULL) {
-            HtpState *htp_state = f->alstate;
-            if (toserver_dir) {
-                htp_state->flags |= HTP_FLAG_STORE_FILES_TS;
-                FileStoreAllFiles(htp_state->files_ts);
+        /* set in flow and AppLayerStateData */
+        AppLayerStateData *sd = AppLayerParserGetStateData(f->proto, f->alproto, f->alstate);
+        if (toclient_dir) {
+            f->file_flags |= FLOWFILE_STORE_TC;
+            if (sd != NULL) {
+                sd->file_flags |= FLOWFILE_STORE_TC;
             }
-            if (toclient_dir) {
-                htp_state->flags |= HTP_FLAG_STORE_FILES_TC;
-                FileStoreAllFiles(htp_state->files_tc);
+        }
+        if (toserver_dir) {
+            f->file_flags |= FLOWFILE_STORE_TS;
+            if (sd != NULL) {
+                sd->file_flags |= FLOWFILE_STORE_TS;
             }
         }
     } else {
@@ -184,10 +199,9 @@ static int FilestorePostMatchWithOptions(Packet *p, Flow *f, const DetectFilesto
  *  When we are sure all parts of the signature matched, we run this function
  *  to finalize the filestore.
  */
-int DetectFilestorePostMatch(ThreadVars *t, DetectEngineThreadCtx *det_ctx, Packet *p, const Signature *s)
+static int DetectFilestorePostMatch(DetectEngineThreadCtx *det_ctx,
+        Packet *p, const Signature *s, const SigMatchCtx *ctx)
 {
-    uint8_t flags = 0;
-
     SCEnter();
 
     if (det_ctx->filestore_cnt == 0) {
@@ -202,33 +216,41 @@ int DetectFilestorePostMatch(ThreadVars *t, DetectEngineThreadCtx *det_ctx, Pack
 #endif
     }
 
-    /* set filestore depth for stream reassembling */
-    TcpSession *ssn = (TcpSession *)p->flow->protoctx;
-    TcpSessionSetReassemblyDepth(ssn, FileReassemblyDepth());
-
-    if (p->flowflags & FLOW_PKT_TOCLIENT)
-        flags |= STREAM_TOCLIENT;
-    else
-        flags |= STREAM_TOSERVER;
-
-    FileContainer *ffc = AppLayerParserGetFiles(p->flow->proto, p->flow->alproto,
-                                                p->flow->alstate, flags);
-
-    /* filestore for single files only */
-    if (s->filestore_ctx == NULL) {
-        uint16_t u;
-        for (u = 0; u < det_ctx->filestore_cnt; u++) {
-            FileStoreFileById(ffc, det_ctx->filestore[u].file_id);
-        }
-    } else {
-        uint16_t u;
-
-        for (u = 0; u < det_ctx->filestore_cnt; u++) {
-            FilestorePostMatchWithOptions(p, p->flow, s->filestore_ctx, ffc,
-                    det_ctx->filestore[u].file_id, det_ctx->filestore[u].tx_id);
-        }
+    if (p->flow->proto == IPPROTO_TCP && p->flow->protoctx != NULL) {
+        /* set filestore depth for stream reassembling */
+        TcpSession *ssn = (TcpSession *)p->flow->protoctx;
+        TcpSessionSetReassemblyDepth(ssn, FileReassemblyDepth());
     }
 
+    SCLogDebug("s->filestore_ctx %p", s->filestore_ctx);
+
+    const uint8_t flags = STREAM_FLAGS_FOR_PACKET(p);
+    for (uint16_t u = 0; u < det_ctx->filestore_cnt; u++) {
+        void *alstate = FlowGetAppState(p->flow);
+        AppLayerParserSetStreamDepthFlag(
+                p->flow->proto, p->flow->alproto, alstate, det_ctx->filestore[u].tx_id, flags);
+
+        void *txv = AppLayerParserGetTx(
+                p->flow->proto, p->flow->alproto, alstate, det_ctx->filestore[u].tx_id);
+        DEBUG_VALIDATE_BUG_ON(txv == NULL);
+        if (txv) {
+            AppLayerGetFileState files = AppLayerParserGetTxFiles(p->flow, txv, flags);
+            FileContainer *ffc_tx = files.fc;
+            DEBUG_VALIDATE_BUG_ON(ffc_tx == NULL);
+            if (ffc_tx) {
+                SCLogDebug("u %u txv %p ffc_tx %p file_id %u", u, txv, ffc_tx,
+                        det_ctx->filestore[u].file_id);
+
+                /* filestore for single files only */
+                if (s->filestore_ctx == NULL) {
+                    FileStoreFileById(ffc_tx, det_ctx->filestore[u].file_id);
+                } else {
+                    FilestorePostMatchWithOptions(p, p->flow, s->filestore_ctx, ffc_tx,
+                            det_ctx->filestore[u].file_id, det_ctx->filestore[u].tx_id);
+                }
+            }
+        }
+    }
     SCReturnInt(0);
 }
 
@@ -249,13 +271,20 @@ int DetectFilestorePostMatch(ThreadVars *t, DetectEngineThreadCtx *det_ctx, Pack
  * \todo when we start supporting more protocols, the logic in this function
  *       needs to be put behind a api.
  */
-static int DetectFilestoreMatch (ThreadVars *t, DetectEngineThreadCtx *det_ctx, Flow *f,
+static int DetectFilestoreMatch (DetectEngineThreadCtx *det_ctx, Flow *f,
         uint8_t flags, File *file, const Signature *s, const SigMatchCtx *m)
 {
     uint32_t file_id = 0;
 
     SCEnter();
 
+    if (!RunmodeIsUnittests()) {
+        extern bool g_filedata_logger_enabled;
+        if (!g_filedata_logger_enabled) {
+            SCLogDebug("not storing file match: no filedata logger enabled");
+            SCReturnInt(1);
+        }
+    }
     if (det_ctx->filestore_cnt >= DETECT_FILESTORE_MAX) {
         SCReturnInt(1);
     }
@@ -263,9 +292,28 @@ static int DetectFilestoreMatch (ThreadVars *t, DetectEngineThreadCtx *det_ctx,
     /* file can be NULL when a rule with filestore scope > file
      * matches. */
     if (file != NULL) {
-        file_id = file->file_store_id;
+        file_id = file->file_track_id;
+        if (file->sid != NULL && s->id > 0) {
+            if (file->sid_cnt >= file->sid_max) {
+                void *p = SCRealloc(file->sid, sizeof(uint32_t) * (file->sid_max + 8));
+                if (p == NULL) {
+                    SCFree(file->sid);
+                    file->sid = NULL;
+                    file->sid_cnt = 0;
+                    file->sid_max = 0;
+                    goto continue_after_realloc_fail;
+                } else {
+                    file->sid = p;
+                    file->sid_max += 8;
+                }
+            }
+            file->sid[file->sid_cnt] = s->id;
+            file->sid_cnt++;
+        }
     }
 
+continue_after_realloc_fail:
+
     det_ctx->filestore[det_ctx->filestore_cnt].file_id = file_id;
     det_ctx->filestore[det_ctx->filestore_cnt].tx_id = det_ctx->tx_id;
 
@@ -292,66 +340,85 @@ static int DetectFilestoreSetup (DetectEngineCtx *de_ctx, Signature *s, const ch
 {
     SCEnter();
 
-    DetectFilestoreData *fd = NULL;
-    SigMatch *sm = NULL;
-    char *args[3] = {NULL,NULL,NULL};
-#define MAX_SUBSTRINGS 30
-    int ret = 0, res = 0;
-    int ov[MAX_SUBSTRINGS];
+    static bool warn_not_configured = false;
+    static uint32_t de_version = 0;
 
-    /* filestore and bypass keywords can't work together */
-    if (s->flags & SIG_FLAG_BYPASS) {
-        SCLogError(SC_ERR_CONFLICTING_RULE_KEYWORDS,
-                   "filestore can't work with bypass keyword");
+    if (de_ctx->filestore_cnt == UINT16_MAX) {
+        SCLogError("Cannot have more than 65535 filestore signatures");
         return -1;
     }
 
-    sm = SigMatchAlloc();
-    if (sm == NULL)
-        goto error;
+    /* Check on first-time loads (includes following a reload) */
+    if (!warn_not_configured || (de_ctx->version != de_version)) {
+        if (de_version != de_ctx->version) {
+            SCLogDebug("reload-detected; re-checking feature presence; DE version now %"PRIu32,
+                       de_ctx->version);
+        }
+        if (!RequiresFeature(FEATURE_OUTPUT_FILESTORE)) {
+            SCLogWarning("One or more rule(s) depends on the "
+                         "file-store output log which is not enabled. "
+                         "Enable the output \"file-store\".");
+        }
+        warn_not_configured = true;
+        de_version = de_ctx->version;
+    }
 
-    sm->type = DETECT_FILESTORE;
+    DetectFilestoreData *fd = NULL;
+    char *args[3] = {NULL,NULL,NULL};
+    int res = 0;
+    size_t pcre2len;
+    pcre2_match_data *match = NULL;
+
+    /* filestore and bypass keywords can't work together */
+    if (s->flags & SIG_FLAG_BYPASS) {
+        SCLogError("filestore can't work with bypass keyword");
+        return -1;
+    }
 
     if (str != NULL && strlen(str) > 0) {
+        char str_0[32];
+        char str_1[32];
+        char str_2[32];
         SCLogDebug("str %s", str);
 
-        ret = pcre_exec(parse_regex, parse_regex_study, str, strlen(str), 0, 0, ov, MAX_SUBSTRINGS);
+        int ret = DetectParsePcreExec(&parse_regex, &match, str, 0, 0);
         if (ret < 1 || ret > 4) {
-            SCLogError(SC_ERR_PCRE_MATCH, "parse error, ret %" PRId32 ", string %s", ret, str);
+            SCLogError("parse error, ret %" PRId32 ", string %s", ret, str);
             goto error;
         }
 
         if (ret > 1) {
-            const char *str_ptr;
-            res = pcre_get_substring((char *)str, ov, MAX_SUBSTRINGS, 1, &str_ptr);
+            pcre2len = sizeof(str_0);
+            res = pcre2_substring_copy_bynumber(match, 1, (PCRE2_UCHAR8 *)str_0, &pcre2len);
             if (res < 0) {
-                SCLogError(SC_ERR_PCRE_GET_SUBSTRING, "pcre_get_substring failed");
+                SCLogError("pcre2_substring_copy_bynumber failed");
                 goto error;
             }
-            args[0] = (char *)str_ptr;
+            args[0] = (char *)str_0;
 
             if (ret > 2) {
-                res = pcre_get_substring((char *)str, ov, MAX_SUBSTRINGS, 2, &str_ptr);
+                pcre2len = sizeof(str_1);
+                res = pcre2_substring_copy_bynumber(match, 2, (PCRE2_UCHAR8 *)str_1, &pcre2len);
                 if (res < 0) {
-                    SCLogError(SC_ERR_PCRE_GET_SUBSTRING, "pcre_get_substring failed");
+                    SCLogError("pcre2_substring_copy_bynumber failed");
                     goto error;
                 }
-                args[1] = (char *)str_ptr;
+                args[1] = (char *)str_1;
             }
             if (ret > 3) {
-                res = pcre_get_substring((char *)str, ov, MAX_SUBSTRINGS, 3, &str_ptr);
+                pcre2len = sizeof(str_2);
+                res = pcre2_substring_copy_bynumber(match, 3, (PCRE2_UCHAR8 *)str_2, &pcre2len);
                 if (res < 0) {
-                    SCLogError(SC_ERR_PCRE_GET_SUBSTRING, "pcre_get_substring failed");
+                    SCLogError("pcre2_substring_copy_bynumber failed");
                     goto error;
                 }
-                args[2] = (char *)str_ptr;
+                args[2] = (char *)str_2;
             }
         }
 
-        fd = SCMalloc(sizeof(DetectFilestoreData));
+        fd = SCCalloc(1, sizeof(DetectFilestoreData));
         if (unlikely(fd == NULL))
             goto error;
-        memset(fd, 0x00, sizeof(DetectFilestoreData));
 
         if (args[0] != NULL) {
             SCLogDebug("first arg %s", args[0]);
@@ -395,29 +462,40 @@ static int DetectFilestoreSetup (DetectEngineCtx *de_ctx, Signature *s, const ch
             if (fd->scope == 0)
                 fd->scope = FILESTORE_SCOPE_DEFAULT;
         }
-
-        sm->ctx = (SigMatchCtx*)fd;
-    } else {
-        sm->ctx = (SigMatchCtx*)NULL;
     }
 
-    if (s->alproto == ALPROTO_HTTP) {
+    if (s->alproto == ALPROTO_HTTP1 || s->alproto == ALPROTO_HTTP) {
         AppLayerHtpNeedFileInspection();
     }
 
-    SigMatchAppendSMToList(s, sm, g_file_match_list_id);
-    s->filestore_ctx = (const DetectFilestoreData *)sm->ctx;
+    if (SCSigMatchAppendSMToList(
+                de_ctx, s, DETECT_FILESTORE, (SigMatchCtx *)fd, g_file_match_list_id) == NULL) {
+        DetectFilestoreFree(de_ctx, fd);
+        goto error;
+    }
+    s->filestore_ctx = fd;
+
+    if (SCSigMatchAppendSMToList(
+                de_ctx, s, DETECT_FILESTORE_POSTMATCH, NULL, DETECT_SM_LIST_POSTMATCH) == NULL) {
+        goto error;
+    }
 
     s->flags |= SIG_FLAG_FILESTORE;
+    de_ctx->filestore_cnt++;
+
+    if (match)
+        pcre2_match_data_free(match);
+
     return 0;
 
 error:
-    if (sm != NULL)
-        SCFree(sm);
+    if (match) {
+        pcre2_match_data_free(match);
+    }
     return -1;
 }
 
-static void DetectFilestoreFree(void *ptr)
+static void DetectFilestoreFree(DetectEngineCtx *de_ctx, void *ptr)
 {
     if (ptr != NULL) {
         SCFree(ptr);
@@ -450,11 +528,9 @@ static int DetectFilestoreTest01(void)
 
     return result;
 }
-#endif /* UNITTESTS */
 
 void DetectFilestoreRegisterTests(void)
 {
-#ifdef UNITTESTS
     UtRegisterTest("DetectFilestoreTest01", DetectFilestoreTest01);
-#endif /* UNITTESTS */
 }
+#endif /* UNITTESTS */
