@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Open Information Security Foundation
+/* Copyright (C) 2012-2019 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -19,24 +19,25 @@
  * \file
  *
  * \author Ignacio Sanchez <sanchezmartin.ji@gmail.com>
+ * \author Bill Meeks <billmeeks8@gmail.com>
  */
 
-#ifndef __DETECT_GEOIP_H__
-#define __DETECT_GEOIP_H__
+#ifndef SURICATA_DETECT_GEOIP_H
+#define SURICATA_DETECT_GEOIP_H
 
 #ifdef HAVE_GEOIP
 
-#include <GeoIP.h>
-#include "util-spm-bm.h"
+#include <maxminddb.h>
 
 #define GEOOPTION_MAXSIZE 3 /* Country Code (2 chars) + NULL */
 #define GEOOPTION_MAXLOCATIONS 64
 
 typedef struct DetectGeoipData_ {
     uint8_t location[GEOOPTION_MAXLOCATIONS][GEOOPTION_MAXSIZE];  /** country code for now, null term.*/
-    int nlocations; /** number of location strings parsed */
+    int nlocations;  /** number of location strings parsed */
     uint32_t flags;
-    GeoIP *geoengine;
+    int mmdb_status; /** Status of DB open call, MMDB_SUCCESS or error */
+    MMDB_s mmdb;     /** MaxMind DB file handle structure */
 } DetectGeoipData;
 
 #endif
