@@ -23,12 +23,8 @@
  * Implements the gid keyword
  */
 
-#ifndef __DETECT_GID_H__
-#define __DETECT_GID_H__
-
-#include "decode-events.h"
-#include "decode-ipv4.h"
-#include "decode-tcp.h"
+#ifndef SURICATA_DETECT_GID_H
+#define SURICATA_DETECT_GID_H
 
 /**
  * Registration function for gid: keyword
@@ -36,10 +32,4 @@
 
 void DetectGidRegister (void);
 
-/**
- * This function registers unit tests for Gid
- */
-
-void GidRegisterTests(void);
-
-#endif /*__DETECT_GID_H__ */
+#endif /*SURICATA_DETECT_GID_H */
\ No newline at end of file
