@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2013 Open Information Security Foundation
+/* Copyright (C) 2007-2020 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -24,12 +24,13 @@
  */
 
 #include "suricata-common.h"
-#include "debug.h"
 #include "decode.h"
 
 #include "detect.h"
 #include "detect-parse.h"
 #include "detect-engine-prefilter-common.h"
+#include "detect-engine-build.h"
+#include "detect-engine-alert.h"
 
 #include "detect-icmp-id.h"
 
@@ -40,16 +41,17 @@
 
 #define PARSE_REGEX "^\\s*(\"\\s*)?([0-9]+)(\\s*\")?\\s*$"
 
-static pcre *parse_regex;
-static pcre_extra *parse_regex_study;
+static DetectParseRegex parse_regex;
 
-static int DetectIcmpIdMatch(ThreadVars *, DetectEngineThreadCtx *, Packet *,
+static int DetectIcmpIdMatch(DetectEngineThreadCtx *, Packet *,
         const Signature *, const SigMatchCtx *);
 static int DetectIcmpIdSetup(DetectEngineCtx *, Signature *, const char *);
-void DetectIcmpIdRegisterTests(void);
-void DetectIcmpIdFree(void *);
+#ifdef UNITTESTS
+static void DetectIcmpIdRegisterTests(void);
+#endif
+void DetectIcmpIdFree(DetectEngineCtx *, void *);
 static int PrefilterSetupIcmpId(DetectEngineCtx *de_ctx, SigGroupHead *sgh);
-static _Bool PrefilterIcmpIdIsPrefilterable(const Signature *s);
+static bool PrefilterIcmpIdIsPrefilterable(const Signature *s);
 
 /**
  * \brief Registration function for icode: icmp_id
@@ -57,27 +59,25 @@ static _Bool PrefilterIcmpIdIsPrefilterable(const Signature *s);
 void DetectIcmpIdRegister (void)
 {
     sigmatch_table[DETECT_ICMP_ID].name = "icmp_id";
-    sigmatch_table[DETECT_ICMP_ID].desc = "check for a ICMP id";
-    sigmatch_table[DETECT_ICMP_ID].url = DOC_URL DOC_VERSION "/rules/header-keywords.html#icmp-id";
+    sigmatch_table[DETECT_ICMP_ID].desc = "check for a ICMP ID";
+    sigmatch_table[DETECT_ICMP_ID].url = "/rules/header-keywords.html#icmp-id";
     sigmatch_table[DETECT_ICMP_ID].Match = DetectIcmpIdMatch;
     sigmatch_table[DETECT_ICMP_ID].Setup = DetectIcmpIdSetup;
     sigmatch_table[DETECT_ICMP_ID].Free = DetectIcmpIdFree;
+#ifdef UNITTESTS
     sigmatch_table[DETECT_ICMP_ID].RegisterTests = DetectIcmpIdRegisterTests;
-
+#endif
     sigmatch_table[DETECT_ICMP_ID].SupportsPrefilter = PrefilterIcmpIdIsPrefilterable;
     sigmatch_table[DETECT_ICMP_ID].SetupPrefilter = PrefilterSetupIcmpId;
 
-    DetectSetupParseRegexes(PARSE_REGEX, &parse_regex, &parse_regex_study);
+    DetectSetupParseRegexes(PARSE_REGEX, &parse_regex);
 }
 
-static inline _Bool GetIcmpId(Packet *p, uint16_t *id)
+static inline bool GetIcmpId(Packet *p, uint16_t *id)
 {
-    if (PKT_IS_PSEUDOPKT(p))
-        return FALSE;
-
     uint16_t pid;
-    if (PKT_IS_ICMPV4(p)) {
-        switch (ICMPV4_GET_TYPE(p)){
+    if (PacketIsICMPv4(p)) {
+        switch (p->icmp_s.type) {
             case ICMP_ECHOREPLY:
             case ICMP_ECHO:
             case ICMP_TIMESTAMP:
@@ -94,10 +94,10 @@ static inline _Bool GetIcmpId(Packet *p, uint16_t *id)
                 break;
             default:
                 SCLogDebug("Packet has no id field");
-                return FALSE;
+                return false;
         }
-    } else if (PKT_IS_ICMPV6(p)) {
-        switch (ICMPV6_GET_TYPE(p)) {
+    } else if (PacketIsICMPv6(p)) {
+        switch (ICMPV6_GET_TYPE(PacketGetICMPv6(p))) {
             case ICMP6_ECHO_REQUEST:
             case ICMP6_ECHO_REPLY:
                 SCLogDebug("ICMPV6_GET_ID(p) %"PRIu16" (network byte order), "
@@ -108,15 +108,15 @@ static inline _Bool GetIcmpId(Packet *p, uint16_t *id)
                 break;
             default:
                 SCLogDebug("Packet has no id field");
-                return FALSE;
+                return false;
         }
     } else {
         SCLogDebug("Packet not ICMPV4 nor ICMPV6");
-        return FALSE;
+        return false;
     }
 
     *id = pid;
-    return TRUE;
+    return true;
 }
 
 /**
@@ -130,12 +130,12 @@ static inline _Bool GetIcmpId(Packet *p, uint16_t *id)
  * \retval 0 no match
  * \retval 1 match
  */
-static int DetectIcmpIdMatch (ThreadVars *t, DetectEngineThreadCtx *det_ctx, Packet *p,
+static int DetectIcmpIdMatch (DetectEngineThreadCtx *det_ctx, Packet *p,
         const Signature *s, const SigMatchCtx *ctx)
 {
     uint16_t pid;
 
-    if (GetIcmpId(p, &pid) == FALSE)
+    if (!GetIcmpId(p, &pid))
         return 0;
 
     const DetectIcmpIdData *iid = (const DetectIcmpIdData *)ctx;
@@ -148,31 +148,32 @@ static int DetectIcmpIdMatch (ThreadVars *t, DetectEngineThreadCtx *det_ctx, Pac
 /**
  * \brief This function is used to parse icmp_id option passed via icmp_id: keyword
  *
+ * \param de_ctx Pointer to the detection engine context
  * \param icmpidstr Pointer to the user provided icmp_id options
  *
  * \retval iid pointer to DetectIcmpIdData on success
  * \retval NULL on failure
  */
-static DetectIcmpIdData *DetectIcmpIdParse (const char *icmpidstr)
+static DetectIcmpIdData *DetectIcmpIdParse (DetectEngineCtx *de_ctx, const char *icmpidstr)
 {
     DetectIcmpIdData *iid = NULL;
     char *substr[3] = {NULL, NULL, NULL};
-#define MAX_SUBSTRINGS 30
-    int ret = 0, res = 0;
-    int ov[MAX_SUBSTRINGS];
+    int res = 0;
+    size_t pcre2_len;
 
-    ret = pcre_exec(parse_regex, parse_regex_study, icmpidstr, strlen(icmpidstr), 0, 0, ov, MAX_SUBSTRINGS);
+    pcre2_match_data *match = NULL;
+    int ret = DetectParsePcreExec(&parse_regex, &match, icmpidstr, 0, 0);
     if (ret < 1 || ret > 4) {
-        SCLogError(SC_ERR_PCRE_MATCH, "Parse error %s", icmpidstr);
+        SCLogError("Parse error %s", icmpidstr);
         goto error;
     }
 
     int i;
     const char *str_ptr;
     for (i = 1; i < ret; i++) {
-        res = pcre_get_substring((char *)icmpidstr, ov, MAX_SUBSTRINGS, i, &str_ptr);
+        res = SC_Pcre2SubstringGet(match, i, (PCRE2_UCHAR8 **)&str_ptr, &pcre2_len);
         if (res < 0) {
-            SCLogError(SC_ERR_PCRE_GET_SUBSTRING, "pcre_get_substring failed");
+            SCLogError("pcre2_substring_get_bynumber failed");
             goto error;
         }
         substr[i-1] = (char *)str_ptr;
@@ -185,35 +186,41 @@ static DetectIcmpIdData *DetectIcmpIdParse (const char *icmpidstr)
 
     if (substr[0]!= NULL && strlen(substr[0]) != 0) {
         if (substr[2] == NULL) {
-            SCLogError(SC_ERR_INVALID_ARGUMENT, "Missing close quote in input");
+            SCLogError("Missing close quote in input");
             goto error;
         }
     } else {
         if (substr[2] != NULL) {
-            SCLogError(SC_ERR_INVALID_ARGUMENT, "Missing open quote in input");
+            SCLogError("Missing open quote in input");
             goto error;
         }
     }
 
-    /** \todo can ByteExtractStringUint16 do this? */
     uint16_t id = 0;
-    if (ByteExtractStringUint16(&id, 10, 0, substr[1]) < 0) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "specified icmp id %s is not "
-                                        "valid", substr[1]);
+    if (StringParseUint16(&id, 10, 0, substr[1]) < 0) {
+        SCLogError("specified icmp id %s is not "
+                   "valid",
+                substr[1]);
         goto error;
     }
     iid->id = htons(id);
 
     for (i = 0; i < 3; i++) {
-        if (substr[i] != NULL) SCFree(substr[i]);
+        if (substr[i] != NULL)
+            pcre2_substring_free((PCRE2_UCHAR8 *)substr[i]);
     }
+    pcre2_match_data_free(match);
     return iid;
 
 error:
-    for (i = 0; i < 3; i++) {
-        if (substr[i] != NULL) SCFree(substr[i]);
+    if (match) {
+        pcre2_match_data_free(match);
     }
-    if (iid != NULL) DetectIcmpIdFree(iid);
+    for (i = 0; i < 3; i++) {
+        if (substr[i] != NULL)
+            pcre2_substring_free((PCRE2_UCHAR8 *)substr[i]);
+    }
+    if (iid != NULL) DetectIcmpIdFree(de_ctx, iid);
     return NULL;
 
 }
@@ -231,25 +238,21 @@ error:
 static int DetectIcmpIdSetup (DetectEngineCtx *de_ctx, Signature *s, const char *icmpidstr)
 {
     DetectIcmpIdData *iid = NULL;
-    SigMatch *sm = NULL;
 
-    iid = DetectIcmpIdParse(icmpidstr);
+    iid = DetectIcmpIdParse(de_ctx, icmpidstr);
     if (iid == NULL) goto error;
 
-    sm = SigMatchAlloc();
-    if (sm == NULL) goto error;
-
-    sm->type = DETECT_ICMP_ID;
-    sm->ctx = (SigMatchCtx *)iid;
-
-    SigMatchAppendSMToList(s, sm, DETECT_SM_LIST_MATCH);
+    if (SCSigMatchAppendSMToList(
+                de_ctx, s, DETECT_ICMP_ID, (SigMatchCtx *)iid, DETECT_SM_LIST_MATCH) == NULL) {
+        goto error;
+    }
     s->flags |= SIG_FLAG_REQUIRE_PACKET;
 
     return 0;
 
 error:
-    if (iid != NULL) DetectIcmpIdFree(iid);
-    if (sm != NULL) SCFree(sm);
+    if (iid != NULL)
+        DetectIcmpIdFree(de_ctx, iid);
     return -1;
 
 }
@@ -259,7 +262,7 @@ error:
  *
  * \param ptr pointer to DetectIcmpIdData
  */
-void DetectIcmpIdFree (void *ptr)
+void DetectIcmpIdFree (DetectEngineCtx *de_ctx, void *ptr)
 {
     DetectIcmpIdData *iid = (DetectIcmpIdData *)ptr;
     SCFree(iid);
@@ -273,7 +276,7 @@ PrefilterPacketIcmpIdMatch(DetectEngineThreadCtx *det_ctx, Packet *p, const void
     const PrefilterPacketHeaderCtx *ctx = pectx;
 
     uint16_t pid;
-    if (GetIcmpId(p, &pid) == FALSE)
+    if (!GetIcmpId(p, &pid))
         return;
 
     if (pid == ctx->v1.u16[0])
@@ -290,33 +293,31 @@ PrefilterPacketIcmpIdSet(PrefilterPacketHeaderValue *v, void *smctx)
     v->u16[0] = a->id;
 }
 
-static _Bool
+static bool
 PrefilterPacketIcmpIdCompare(PrefilterPacketHeaderValue v, void *smctx)
 {
     const DetectIcmpIdData *a = smctx;
     if (v.u16[0] == a->id)
-        return TRUE;
-    return FALSE;
+        return true;
+    return false;
 }
 
 static int PrefilterSetupIcmpId(DetectEngineCtx *de_ctx, SigGroupHead *sgh)
 {
-    return PrefilterSetupPacketHeader(de_ctx, sgh, DETECT_ICMP_ID,
-        PrefilterPacketIcmpIdSet,
-        PrefilterPacketIcmpIdCompare,
-        PrefilterPacketIcmpIdMatch);
+    return PrefilterSetupPacketHeader(de_ctx, sgh, DETECT_ICMP_ID, SIG_MASK_REQUIRE_REAL_PKT,
+            PrefilterPacketIcmpIdSet, PrefilterPacketIcmpIdCompare, PrefilterPacketIcmpIdMatch);
 }
 
-static _Bool PrefilterIcmpIdIsPrefilterable(const Signature *s)
+static bool PrefilterIcmpIdIsPrefilterable(const Signature *s)
 {
     const SigMatch *sm;
     for (sm = s->init_data->smlists[DETECT_SM_LIST_MATCH] ; sm != NULL; sm = sm->next) {
         switch (sm->type) {
             case DETECT_ICMP_ID:
-                return TRUE;
+                return true;
         }
     }
-    return FALSE;
+    return false;
 }
 
 #ifdef UNITTESTS
@@ -328,13 +329,11 @@ static _Bool PrefilterIcmpIdIsPrefilterable(const Signature *s)
  */
 static int DetectIcmpIdParseTest01 (void)
 {
-    DetectIcmpIdData *iid = NULL;
-    iid = DetectIcmpIdParse("300");
-    if (iid != NULL && iid->id == htons(300)) {
-        DetectIcmpIdFree(iid);
-        return 1;
-    }
-    return 0;
+    DetectIcmpIdData *iid = DetectIcmpIdParse(NULL, "300");
+    FAIL_IF_NULL(iid);
+    FAIL_IF_NOT(iid->id == htons(300));
+    DetectIcmpIdFree(NULL, iid);
+    PASS;
 }
 
 /**
@@ -343,13 +342,11 @@ static int DetectIcmpIdParseTest01 (void)
  */
 static int DetectIcmpIdParseTest02 (void)
 {
-    DetectIcmpIdData *iid = NULL;
-    iid = DetectIcmpIdParse("  300  ");
-    if (iid != NULL && iid->id == htons(300)) {
-        DetectIcmpIdFree(iid);
-        return 1;
-    }
-    return 0;
+    DetectIcmpIdData *iid = DetectIcmpIdParse(NULL, "  300  ");
+    FAIL_IF_NULL(iid);
+    FAIL_IF_NOT(iid->id == htons(300));
+    DetectIcmpIdFree(NULL, iid);
+    PASS;
 }
 
 /**
@@ -358,13 +355,11 @@ static int DetectIcmpIdParseTest02 (void)
  */
 static int DetectIcmpIdParseTest03 (void)
 {
-    DetectIcmpIdData *iid = NULL;
-    iid = DetectIcmpIdParse("\"300\"");
-    if (iid != NULL && iid->id == htons(300)) {
-        DetectIcmpIdFree(iid);
-        return 1;
-    }
-    return 0;
+    DetectIcmpIdData *iid = DetectIcmpIdParse(NULL, "\"300\"");
+    FAIL_IF_NULL(iid);
+    FAIL_IF_NOT(iid->id == htons(300));
+    DetectIcmpIdFree(NULL, iid);
+    PASS;
 }
 
 /**
@@ -373,13 +368,11 @@ static int DetectIcmpIdParseTest03 (void)
  */
 static int DetectIcmpIdParseTest04 (void)
 {
-    DetectIcmpIdData *iid = NULL;
-    iid = DetectIcmpIdParse("   \"   300 \"");
-    if (iid != NULL && iid->id == htons(300)) {
-        DetectIcmpIdFree(iid);
-        return 1;
-    }
-    return 0;
+    DetectIcmpIdData *iid = DetectIcmpIdParse(NULL, "   \"   300 \"");
+    FAIL_IF_NULL(iid);
+    FAIL_IF_NOT(iid->id == htons(300));
+    DetectIcmpIdFree(NULL, iid);
+    PASS;
 }
 
 /**
@@ -388,13 +381,9 @@ static int DetectIcmpIdParseTest04 (void)
  */
 static int DetectIcmpIdParseTest05 (void)
 {
-    DetectIcmpIdData *iid = NULL;
-    iid = DetectIcmpIdParse("\"300");
-    if (iid == NULL) {
-        DetectIcmpIdFree(iid);
-        return 1;
-    }
-    return 0;
+    DetectIcmpIdData *iid = DetectIcmpIdParse(NULL, "\"300");
+    FAIL_IF_NOT_NULL(iid);
+    PASS;
 }
 
 /**
@@ -413,7 +402,7 @@ static int DetectIcmpIdMatchTest01 (void)
     memset(&th_v, 0, sizeof(ThreadVars));
 
     p = UTHBuildPacket(NULL, 0, IPPROTO_ICMP);
-    p->icmpv4vars.id = htons(21781);
+    p->l4.vars.icmpv4.id = htons(21781);
 
     DetectEngineCtx *de_ctx = DetectEngineCtxInit();
     if (de_ctx == NULL) {
@@ -496,9 +485,9 @@ static int DetectIcmpIdMatchTest02 (void)
 
     ip4h.s_ip_src.s_addr = p->src.addr_data32[0];
     ip4h.s_ip_dst.s_addr = p->dst.addr_data32[0];
-    p->ip4h = &ip4h;
+    UTHSetIPV4Hdr(p, &ip4h);
 
-    DecodeICMPV4(&th_v, &dtv, p, raw_icmpv4, sizeof(raw_icmpv4), NULL);
+    DecodeICMPV4(&th_v, &dtv, p, raw_icmpv4, sizeof(raw_icmpv4));
 
     DetectEngineCtx *de_ctx = DetectEngineCtxInit();
     if (de_ctx == NULL) {
@@ -535,11 +524,9 @@ end:
     SCFree(p);
     return result;
 }
-#endif /* UNITTESTS */
 
-void DetectIcmpIdRegisterTests (void)
+static void DetectIcmpIdRegisterTests (void)
 {
-#ifdef UNITTESTS
     UtRegisterTest("DetectIcmpIdParseTest01", DetectIcmpIdParseTest01);
     UtRegisterTest("DetectIcmpIdParseTest02", DetectIcmpIdParseTest02);
     UtRegisterTest("DetectIcmpIdParseTest03", DetectIcmpIdParseTest03);
@@ -547,6 +534,5 @@ void DetectIcmpIdRegisterTests (void)
     UtRegisterTest("DetectIcmpIdParseTest05", DetectIcmpIdParseTest05);
     UtRegisterTest("DetectIcmpIdMatchTest01", DetectIcmpIdMatchTest01);
     UtRegisterTest("DetectIcmpIdMatchTest02", DetectIcmpIdMatchTest02);
-#endif /* UNITTESTS */
 }
-
+#endif /* UNITTESTS */
