@@ -21,11 +21,8 @@
  * \author Breno Silva <breno.silva@gmail.com>
  */
 
-#ifndef __DETECT_IPOPTS_H__
-#define __DETECT_IPOPTS_H__
-
-#include "decode-events.h"
-#include "decode-ipv4.h"
+#ifndef SURICATA_DETECT_IPOPTS_H
+#define SURICATA_DETECT_IPOPTS_H
 
 /**
  * \struct DetectIpOptsData_
@@ -47,5 +44,6 @@ typedef struct DetectIpOptsData_ {
 
 void DetectIpOptsRegister (void);
 
-#endif /*__DETECT_IPOPTS_H__ */
+const char *IpOptsFlagToString(uint16_t flag);
 
+#endif /*SURICATA_DETECT_IPOPTS_H */
