@@ -21,8 +21,8 @@
  * \author Brian Rectanus <brectanu@gmail.com>
  */
 
-#ifndef __DETECT_IPPROTO_H__
-#define __DETECT_IPPROTO_H__
+#ifndef SURICATA_DETECT_IPPROTO_H
+#define SURICATA_DETECT_IPPROTO_H
 
 /** IPProto Operators */
 #define DETECT_IPPROTO_OP_EQ     '=' /**< "equals" operator (default) */
@@ -42,7 +42,6 @@ typedef struct DetectIPProtoData_ {
  * \brief Registration function for ip_proto keyword.
  */
 void DetectIPProtoRegister (void);
-void DetectIPProtoRemoveAllSMs(Signature *);
-
-#endif /* __DETECT_IPPROTO_H__ */
+void DetectIPProtoRemoveAllSMs(DetectEngineCtx *, Signature *);
 
+#endif /* SURICATA_DETECT_IPPROTO_H */
