@@ -23,7 +23,7 @@
  * Implements metadata keyword support
  *
  * \todo Do we need to do anything more this is used in snort host attribute table
- *       It is also used for rule managment.
+ *       It is also used for rule management.
  */
 
 #include "suricata-common.h"
@@ -31,21 +31,27 @@
 #include "detect-parse.h"
 #include "detect-engine.h"
 #include "detect-metadata.h"
-
+#include "util-hash-string.h"
 #include "util-unittest.h"
+#include "rust.h"
+#include "util-validate.h"
 
 static int DetectMetadataSetup (DetectEngineCtx *, Signature *, const char *);
+#ifdef UNITTESTS
 static void DetectMetadataRegisterTests(void);
+#endif
 
 void DetectMetadataRegister (void)
 {
     sigmatch_table[DETECT_METADATA].name = "metadata";
-    sigmatch_table[DETECT_METADATA].desc = "used by suricata for logging";
-    sigmatch_table[DETECT_METADATA].url = DOC_URL DOC_VERSION "/rules/meta.html#metadata";
+    sigmatch_table[DETECT_METADATA].desc = "used for logging";
+    sigmatch_table[DETECT_METADATA].url = "/rules/meta.html#metadata";
     sigmatch_table[DETECT_METADATA].Match = NULL;
     sigmatch_table[DETECT_METADATA].Setup = DetectMetadataSetup;
     sigmatch_table[DETECT_METADATA].Free  = NULL;
+#ifdef UNITTESTS
     sigmatch_table[DETECT_METADATA].RegisterTests = DetectMetadataRegisterTests;
+#endif
 }
 
 /**
@@ -60,38 +66,6 @@ void DetectMetadataFree(DetectMetadata *mdata)
     SCReturn;
 }
 
-/* djb2 string hashing */
-static uint32_t StringHashFunc(HashTable *ht, void *data, uint16_t datalen)
-{
-    uint32_t hash = 5381;
-    int c;
-
-    while ((c = *(char *)data++))
-        hash = ((hash << 5) + hash) + c; /* hash * 33 + c */
-
-    hash = hash % ht->array_size;
-
-    return hash;
-}
-
-static char StringHashCompareFunc(void *data1, uint16_t datalen1,
-                               void *data2, uint16_t datalen2)
-{
-    int len1 = strlen((char *)data1);
-    int len2 = strlen((char *)data2);
-
-    if (len1 == len2 && memcmp(data1, data2, len1) == 0) {
-        return 1;
-    }
-
-    return 0;
-}
-
-static void StringHashFreeFunc(void *data)
-{
-    SCFree(data);
-}
-
 int DetectMetadataHashInit(DetectEngineCtx *de_ctx)
 {
     if (! DetectEngineMustParseMetadata())
@@ -109,9 +83,10 @@ void DetectMetadataHashFree(DetectEngineCtx *de_ctx)
         HashTableFree(de_ctx->metadata_table);
 }
 
-static const char *DetectMedatataHashAdd(DetectEngineCtx *de_ctx, const char *string)
+static const char *DetectMetadataHashAdd(DetectEngineCtx *de_ctx, const char *string)
 {
-    const char * hstring = (char *)HashTableLookup(de_ctx->metadata_table, (void *)string, strlen(string));
+    const char *hstring = (char *)HashTableLookup(
+            de_ctx->metadata_table, (void *)string, (uint16_t)strlen(string));
     if (hstring) {
         return hstring;
     }
@@ -121,16 +96,91 @@ static const char *DetectMedatataHashAdd(DetectEngineCtx *de_ctx, const char *st
         return NULL;
     }
 
-    if (HashTableAdd(de_ctx->metadata_table, (void *)astring, strlen(astring)) == 0) {
-        return (char *)HashTableLookup(de_ctx->metadata_table, (void *)astring, strlen(astring));
+    if (HashTableAdd(de_ctx->metadata_table, (void *)astring, (uint16_t)strlen(astring)) == 0) {
+        return (char *)HashTableLookup(
+                de_ctx->metadata_table, (void *)astring, (uint16_t)strlen(astring));
     } else {
         SCFree((void *)astring);
     }
     return NULL;
 }
 
+static int SortHelper(const void *a, const void *b)
+{
+    const DetectMetadata *ma = *(const DetectMetadata **)a;
+    const DetectMetadata *mb = *(const DetectMetadata **)b;
+    return strcasecmp(ma->key, mb->key);
+}
+
+static char *CraftPreformattedJSON(const DetectMetadata *head)
+{
+    int cnt = 0;
+    for (const DetectMetadata *m = head; m != NULL; m = m->next) {
+        cnt++;
+    }
+    if (cnt == 0)
+        return NULL;
+
+    const DetectMetadata *array[cnt];
+    int i = 0;
+    for (const DetectMetadata *m = head; m != NULL; m = m->next) {
+        array[i++] = m;
+    }
+    BUG_ON(i != cnt);
+    qsort(array, cnt, sizeof(DetectMetadata *), SortHelper);
+
+    SCJsonBuilder *js = SCJbNewObject();
+    if (js == NULL)
+        return NULL;
+
+    /* array is sorted by key, so we can create a jsonbuilder object
+     * with each key appearing just once with one or more values */
+    bool array_open = false;
+    for (int j = 0; j < cnt; j++) {
+        const DetectMetadata *m = array[j];
+        const DetectMetadata *nm = j + 1 < cnt ? array[j + 1] : NULL;
+        DEBUG_VALIDATE_BUG_ON(m == NULL); // for scan-build
+
+        if (nm && strcasecmp(m->key, nm->key) == 0) {
+            if (!array_open) {
+                SCJbOpenArray(js, m->key);
+                array_open = true;
+            }
+            SCJbAppendString(js, m->value);
+        } else {
+            if (!array_open) {
+                SCJbOpenArray(js, m->key);
+            }
+            SCJbAppendString(js, m->value);
+            SCJbClose(js);
+            array_open = false;
+        }
+    }
+    SCJbClose(js);
+    /* we have a complete json builder. Now store it as a C string */
+    const size_t len = SCJbLen(js);
+#define MD_STR "\"metadata\":"
+#define MD_STR_LEN (sizeof(MD_STR) - 1)
+    char *str = SCMalloc(len + MD_STR_LEN + 1);
+    if (str == NULL) {
+        SCJbFree(js);
+        return NULL;
+    }
+    char *ptr = str;
+    memcpy(ptr, MD_STR, MD_STR_LEN);
+    ptr += MD_STR_LEN;
+    memcpy(ptr, SCJbPtr(js), len);
+    ptr += len;
+    *ptr = '\0';
+#undef MD_STR
+#undef MD_STR_LEN
+    SCJbFree(js);
+    return str;
+}
+
 static int DetectMetadataParse(DetectEngineCtx *de_ctx, Signature *s, const char *metadatastr)
 {
+    DetectMetadata *head = s->metadata ? s->metadata->list : NULL;
     char copy[strlen(metadatastr)+1];
     strlcpy(copy, metadatastr, sizeof(copy));
     char *xsaveptr = NULL;
@@ -155,15 +205,15 @@ static int DetectMetadataParse(DetectEngineCtx *de_ctx, Signature *s, const char
             goto next;
         }
 
-        const char *hkey = DetectMedatataHashAdd(de_ctx, key);
+        const char *hkey = DetectMetadataHashAdd(de_ctx, key);
         if (hkey == NULL) {
-            SCLogError(SC_ERR_MEM_ALLOC, "can't create metadata key");
+            SCLogError("can't create metadata key");
             continue;
         }
 
-        const char *hval = DetectMedatataHashAdd(de_ctx, val);
+        const char *hval = DetectMetadataHashAdd(de_ctx, val);
         if (hval == NULL) {
-            SCLogError(SC_ERR_MEM_ALLOC, "can't create metadata value");
+            SCLogError("can't create metadata value");
             goto next;
         }
 
@@ -175,13 +225,28 @@ static int DetectMetadataParse(DetectEngineCtx *de_ctx, Signature *s, const char
         }
         dkv->key = hkey;
         dkv->value = hval;
-        dkv->next = s->metadata;
-        s->metadata = dkv;
+        dkv->next = head;
+        head = dkv;
 
     next:
         key = strtok_r(NULL, ",", &xsaveptr);
     }
-
+    if (head != NULL) {
+        if (s->metadata == NULL) {
+            s->metadata = SCCalloc(1, sizeof(*s->metadata));
+            if (s->metadata == NULL) {
+                for (DetectMetadata *m = head; m != NULL; ) {
+                    DetectMetadata *next_m = m->next;
+                    DetectMetadataFree(m);
+                    m = next_m;
+                }
+                return -1;
+            }
+        }
+        s->metadata->list = head;
+        SCFree(s->metadata->json_str);
+        s->metadata->json_str = CraftPreformattedJSON(head);
+    }
     return 0;
 }
 
@@ -215,10 +280,7 @@ static int DetectMetadataParseTest01(void)
 static int DetectMetadataParseTest02(void)
 {
     DetectEngineSetParseMetadata();
-    DetectEngineCtx *de_ctx = NULL;
-    DetectMetadata *dm;
-
-    de_ctx = DetectEngineCtxInit();
+    DetectEngineCtx *de_ctx = DetectEngineCtxInit();
     FAIL_IF_NULL(de_ctx);
     Signature *sig = DetectEngineAppendSig(de_ctx,
                                            "alert tcp any any -> any any "
@@ -227,30 +289,28 @@ static int DetectMetadataParseTest02(void)
                                            "sid:1; rev:1;)");
     FAIL_IF_NULL(sig);
     FAIL_IF_NULL(sig->metadata); 
-    FAIL_IF_NULL(sig->metadata->key); 
-    FAIL_IF(strcmp("jaivu", sig->metadata->key));
-    FAIL_IF(strcmp("gros_minet", sig->metadata->value));
-    FAIL_IF_NULL(sig->metadata->next); 
-    dm = sig->metadata->next;
+    FAIL_IF_NULL(sig->metadata->list); 
+    FAIL_IF_NULL(sig->metadata->list->key); 
+    FAIL_IF(strcmp("jaivu", sig->metadata->list->key));
+    FAIL_IF(strcmp("gros_minet", sig->metadata->list->value));
+    FAIL_IF_NULL(sig->metadata->list->next); 
+    DetectMetadata *dm = sig->metadata->list->next;
     FAIL_IF(strcmp("titi", dm->key));
     dm = dm->next;
     FAIL_IF_NULL(dm);
     FAIL_IF(strcmp("toto", dm->key));
-
+    FAIL_IF_NOT(strcmp(sig->metadata->json_str,
+        "\"metadata\":{\"jaivu\":[\"gros_minet\"],\"titi\":[\"2\"],\"toto\":[\"1\"]}") == 0);
     DetectEngineCtxFree(de_ctx);
     PASS;
 }
 
-#endif /* UNITTESTS */
-
 /**
  * \brief this function registers unit tests for DetectCipService
  */
 static void DetectMetadataRegisterTests(void)
 {
-#ifdef UNITTESTS
     UtRegisterTest("DetectMetadataParseTest01", DetectMetadataParseTest01);
     UtRegisterTest("DetectMetadataParseTest02", DetectMetadataParseTest02);
-#endif /* UNITTESTS */
 }
-
+#endif /* UNITTESTS */
