@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2010 Open Information Security Foundation
+/* Copyright (C) 2007-2020 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -43,11 +43,16 @@
 #include "util-byte.h"
 #include "util-debug.h"
 
-#define PARSE_REGEX "^\\s*([A-Za-z0-9]+)\\s*,\"?\\s*\"?\\s*([a-zA-Z0-9\\-_\\.\\/\\?\\=]+)\"?\\s*\"?"
+/* Breakout key and scheme (optional) and domain/path (mandatory) */
+#define PARSE_REGEX                                                                                \
+    "^\\s*([A-Za-z0-9]+)\\s*,\"?\\s*\"?\\s*([a-zA-Z]+:\\/\\/)?([a-zA-Z0-9\\-_\\.\\/"               \
+    "\\?\\=]+)\"?\\s*\"?"
 
-static pcre *parse_regex;
-static pcre_extra *parse_regex_study;
+static DetectParseRegex parse_regex;
 
+#ifdef UNITTESTS
+static void ReferenceRegisterTests(void);
+#endif
 static int DetectReferenceSetup(DetectEngineCtx *, Signature *s, const char *str);
 
 /**
@@ -57,13 +62,12 @@ void DetectReferenceRegister(void)
 {
     sigmatch_table[DETECT_REFERENCE].name = "reference";
     sigmatch_table[DETECT_REFERENCE].desc = "direct to places where information about the rule can be found";
-    sigmatch_table[DETECT_REFERENCE].url = DOC_URL DOC_VERSION "/rules/meta.html#reference";
-    sigmatch_table[DETECT_REFERENCE].Match = NULL;
+    sigmatch_table[DETECT_REFERENCE].url = "/rules/meta.html#reference";
     sigmatch_table[DETECT_REFERENCE].Setup = DetectReferenceSetup;
-    sigmatch_table[DETECT_REFERENCE].Free  = NULL;
+#ifdef UNITTESTS
     sigmatch_table[DETECT_REFERENCE].RegisterTests = ReferenceRegisterTests;
-
-    DetectSetupParseRegexes(PARSE_REGEX, &parse_regex, &parse_regex_study);
+#endif
+    DetectSetupParseRegexes(PARSE_REGEX, &parse_regex);
 }
 
 /**
@@ -73,6 +77,9 @@ void DetectReferenceFree(DetectReference *ref)
 {
     SCEnter();
 
+    if (ref->key)
+        SCFree(ref->key);
+
     if (ref->reference != NULL) {
         SCFree(ref->reference);
     }
@@ -94,66 +101,105 @@ static DetectReference *DetectReferenceParse(const char *rawstr, DetectEngineCtx
 {
     SCEnter();
 
-    DetectReference *ref = NULL;
-#define MAX_SUBSTRINGS 30
-    int ret = 0, res = 0;
-    int ov[MAX_SUBSTRINGS];
-    char key[64] = "";
-    char content[1024] = "";
+    int res = 0;
+    size_t pcre2len;
+    char key[REFERENCE_SYSTEM_NAME_MAX] = "";
+    char scheme[REFERENCE_SYSTEM_NAME_MAX] = "";
+    char uri[REFERENCE_CONTENT_NAME_MAX] = "";
 
-    ret = pcre_exec(parse_regex, parse_regex_study, rawstr, strlen(rawstr),
-                    0, 0, ov, MAX_SUBSTRINGS);
-    if (ret < 2) {
-        SCLogError(SC_ERR_INVALID_SIGNATURE, "Unable to parse \"reference\" "
-                   "keyword argument - \"%s\".   Invalid argument.", rawstr);
-        goto error;
+    pcre2_match_data *match = NULL;
+    int ret = DetectParsePcreExec(&parse_regex, &match, rawstr, 0, 0);
+    if (ret != 4) {
+        SCLogError("Unable to parse \"reference\" "
+                   "keyword argument - \"%s\".   Invalid argument.",
+                rawstr);
+        if (match) {
+            pcre2_match_data_free(match);
+        }
+        return NULL;
     }
 
-    ref = SCMalloc(sizeof(DetectReference));
+    DetectReference *ref = SCCalloc(1, sizeof(DetectReference));
     if (unlikely(ref == NULL)) {
-        goto error;
+        pcre2_match_data_free(match);
+        return NULL;
     }
-    memset(ref, 0, sizeof(DetectReference));
 
-    res = pcre_copy_substring((char *)rawstr, ov, MAX_SUBSTRINGS, 1, key, sizeof(key));
+    /* Position 1 = key (mandatory) */
+    pcre2len = sizeof(key);
+    res = pcre2_substring_copy_bynumber(match, 1, (PCRE2_UCHAR8 *)key, &pcre2len);
     if (res < 0) {
-        SCLogError(SC_ERR_PCRE_GET_SUBSTRING, "pcre_copy_substring failed");
+        SCLogError("pcre2_substring_copy_bynumber key failed");
         goto error;
     }
 
-    res = pcre_copy_substring((char *)rawstr, ov, MAX_SUBSTRINGS, 2, content, sizeof(content));
+    /* Position 2 = scheme (optional) */
+    pcre2len = sizeof(scheme);
+    (void)pcre2_substring_copy_bynumber(match, 2, (PCRE2_UCHAR8 *)scheme, &pcre2len);
+
+    /* Position 3 = domain-path (mandatory) */
+    pcre2len = sizeof(uri);
+    res = pcre2_substring_copy_bynumber(match, 3, (PCRE2_UCHAR8 *)uri, &pcre2len);
     if (res < 0) {
-        SCLogError(SC_ERR_PCRE_GET_SUBSTRING, "pcre_get_substring failed");
+        SCLogError("pcre2_substring_copy_bynumber domain-path failed");
         goto error;
     }
 
-    if (strlen(key) == 0 || strlen(content) == 0)
+    size_t ref_len = strlen(uri);
+    /* no key, reference -- return an error */
+    if (strlen(key) == 0 || ref_len == 0)
         goto error;
 
-    SCRConfReference *lookup_ref_conf = SCRConfGetReference(key, de_ctx);
-    if (lookup_ref_conf != NULL) {
-        ref->key = lookup_ref_conf->url;
+    if (strlen(scheme)) {
+        SCLogConfig("scheme value %s overrides key %s", scheme, key);
+        ref->key = SCStrdup(scheme);
+        /* already bound checked to be REFERENCE_SYSTEM_NAME_MAX or less */
+        ref->key_len = (uint16_t)strlen(scheme);
     } else {
-        SCLogError(SC_ERR_REFERENCE_UNKNOWN, "unknown reference key \"%s\". "
-                   "Supported keys are defined in reference.config file.  Please "
-                   "have a look at the conf param \"reference-config-file\"", key);
-        goto error;
+
+        SCRConfReference *lookup_ref_conf = SCRConfGetReference(key, de_ctx);
+        if (lookup_ref_conf != NULL) {
+            ref->key = SCStrdup(lookup_ref_conf->url);
+            /* already bound checked to be REFERENCE_SYSTEM_NAME_MAX or less */
+            ref->key_len = (uint16_t)strlen(ref->key);
+        } else {
+            if (SigMatchStrictEnabled(DETECT_REFERENCE)) {
+                SCLogError("unknown reference key \"%s\"", key);
+                goto error;
+            }
+
+            SCLogWarning("unknown reference key \"%s\"", key);
+
+            char str[2048];
+            snprintf(str, sizeof(str), "config reference: %s undefined\n", key);
+
+            if (SCRConfAddReference(de_ctx, str) < 0)
+                goto error;
+            lookup_ref_conf = SCRConfGetReference(key, de_ctx);
+            if (lookup_ref_conf == NULL)
+                goto error;
+        }
     }
 
     /* make a copy so we can free pcre's substring */
-    ref->reference = SCStrdup((char *)content);
+    ref->reference = SCStrdup(uri);
     if (ref->reference == NULL) {
-        SCLogError(SC_ERR_MEM_ALLOC, "strdup failed: %s", strerror(errno));
+        SCLogError("strdup failed: %s", strerror(errno));
         goto error;
     }
 
+    /* already bound checked to be REFERENCE_CONTENT_NAME_MAX or less */
+    ref->reference_len = (uint16_t)ref_len;
+
+    pcre2_match_data_free(match);
     /* free the substrings */
     SCReturnPtr(ref, "Reference");
 
 error:
-    if (ref != NULL)
-        DetectReferenceFree(ref);
-
+    if (match) {
+        pcre2_match_data_free(match);
+    }
+    DetectReferenceFree(ref);
     SCReturnPtr(NULL, "Reference");
 }
 
@@ -174,12 +220,11 @@ static int DetectReferenceSetup(DetectEngineCtx *de_ctx, Signature *s,
 {
     SCEnter();
 
-    DetectReference *ref = NULL;
     DetectReference *sig_refs = NULL;
 
-    ref = DetectReferenceParse(rawstr, de_ctx);
+    DetectReference *ref = DetectReferenceParse(rawstr, de_ctx);
     if (ref == NULL)
-        goto error;
+        SCReturnInt(-1);
 
     SCLogDebug("ref %s %s", ref->key, ref->reference);
 
@@ -195,9 +240,6 @@ static int DetectReferenceSetup(DetectEngineCtx *de_ctx, Signature *s,
     }
 
     SCReturnInt(0);
-
-error:
-    SCReturnInt(-1);
 }
 
 /***************************************Unittests******************************/
@@ -207,150 +249,96 @@ error:
 /**
  * \test one valid reference.
  *
- *  \retval 1 on succces.
+ *  \retval 1 on success.
  *  \retval 0 on failure.
  */
 static int DetectReferenceParseTest01(void)
 {
-    int result = 0;
-    Signature *s = NULL;
-    DetectReference *ref = NULL;
-
     DetectEngineCtx *de_ctx = DetectEngineCtxInit();
-    if (de_ctx == NULL) {
-        goto cleanup;
-    }
+    FAIL_IF_NULL(de_ctx);
     de_ctx->flags |= DE_QUIET;
 
     FILE *fd = SCRConfGenerateValidDummyReferenceConfigFD01();
+    FAIL_IF_NULL(fd);
     SCRConfLoadReferenceConfigFile(de_ctx, fd);
 
-    s = de_ctx->sig_list = SigInit(de_ctx, "alert icmp any any -> any any "
-                                   "(msg:\"One reference\"; reference:one,001-2010; sid:2;)");
-    if (s == NULL) {
-        goto cleanup;
-    }
+    Signature *s = DetectEngineAppendSig(de_ctx, "alert icmp any any -> any any "
+            "(msg:\"One reference\"; reference:one,001-2010; sid:2;)");
+    FAIL_IF_NULL(s);
+    FAIL_IF_NULL(s->references);
 
-    if (s->references == NULL)  {
-        goto cleanup;
-    }
-
-    ref = s->references;
-    if (strcmp(ref->key, "http://www.one.com") != 0 ||
-        strcmp(ref->reference, "001-2010") != 0) {
-        goto cleanup;
-    }
-
-    result = 1;
-
-cleanup:
-    if (de_ctx != NULL) {
-        DetectEngineCtxFree(de_ctx);
-    }
-    return result;
+    DetectReference *ref = s->references;
+    FAIL_IF (strcmp(ref->key, "http://www.one.com") != 0);
+    FAIL_IF (strcmp(ref->reference, "001-2010") != 0);
 
+    DetectEngineCtxFree(de_ctx);
+    PASS;
 }
 
 /**
  * \test for two valid references.
  *
- *  \retval 1 on succces.
+ *  \retval 1 on success.
  *  \retval 0 on failure.
  */
 static int DetectReferenceParseTest02(void)
 {
-    int result = 0;
-    Signature *s = NULL;
-
     DetectEngineCtx *de_ctx = DetectEngineCtxInit();
-    if (de_ctx == NULL) {
-        goto cleanup;
-    }
+    FAIL_IF_NULL(de_ctx);
     de_ctx->flags |= DE_QUIET;
 
     FILE *fd = SCRConfGenerateValidDummyReferenceConfigFD01();
+    FAIL_IF_NULL(fd);
     SCRConfLoadReferenceConfigFile(de_ctx, fd);
 
-    s = de_ctx->sig_list = SigInit(de_ctx, "alert icmp any any -> any any "
+    Signature *s = DetectEngineAppendSig(de_ctx, "alert icmp any any -> any any "
                                    "(msg:\"Two references\"; "
                                    "reference:one,openinfosecdoundation.txt; "
                                    "reference:two,001-2010; sid:2;)");
-    if (s == NULL) {
-        printf("sig parse failed: ");
-        goto cleanup;
-    }
+    FAIL_IF_NULL(s);
+    FAIL_IF_NULL(s->references);
+    FAIL_IF_NULL(s->references->next);
 
-    if (s->references == NULL || s->references->next == NULL)  {
-        printf("no ref or not enough refs: ");
-        goto cleanup;
-    }
+    DetectReference *ref = s->references;
+    FAIL_IF (strcmp(ref->key, "http://www.one.com") != 0);
+    FAIL_IF (strcmp(ref->reference, "openinfosecdoundation.txt") != 0);
 
-    if (strcmp(s->references->key, "http://www.one.com") != 0 ||
-        strcmp(s->references->reference, "openinfosecdoundation.txt") != 0) {
-        printf("first ref failed: ");
-        goto cleanup;
-    }
+    ref = s->references->next;
+    FAIL_IF (strcmp(ref->key, "http://www.two.com") != 0);
+    FAIL_IF (strcmp(ref->reference, "001-2010") != 0);
 
-    if (strcmp(s->references->next->key, "http://www.two.com") != 0 ||
-        strcmp(s->references->next->reference, "001-2010") != 0) {
-        printf("second ref failed: ");
-        goto cleanup;
-    }
-
-    result = 1;
-
-cleanup:
-    if (de_ctx != NULL) {
-        DetectEngineCtxFree(de_ctx);
-    }
-    return result;
+    DetectEngineCtxFree(de_ctx);
+    PASS;
 }
 
 /**
  * \test parsing: invalid reference.
  *
- *  \retval 1 on succces.
+ *  \retval 1 on success.
  *  \retval 0 on failure.
  */
 static int DetectReferenceParseTest03(void)
 {
-    int result = 0;
-    Signature *s = NULL;
     DetectEngineCtx *de_ctx = DetectEngineCtxInit();
-    if (de_ctx == NULL) {
-        goto cleanup;
-    }
+    FAIL_IF_NULL(de_ctx);
     de_ctx->flags |= DE_QUIET;
 
-    FILE *fd =SCRConfGenerateValidDummyReferenceConfigFD01();
+    FILE *fd = SCRConfGenerateValidDummyReferenceConfigFD01();
+    FAIL_IF_NULL(fd);
     SCRConfLoadReferenceConfigFile(de_ctx, fd);
 
-    s = de_ctx->sig_list = SigInit(de_ctx, "alert icmp any any -> any any "
+    Signature *s = DetectEngineAppendSig(de_ctx, "alert icmp any any -> any any "
                                    "(msg:\"invalid ref\"; "
                                    "reference:unknownkey,001-2010; sid:2;)");
-    if (s != NULL) {
-        printf("sig parsed even though it's invalid: ");
-        goto cleanup;
-    }
-
-    result = 1;
-
-cleanup:
-    if (de_ctx != NULL) {
-        DetectEngineCtxFree(de_ctx);
-    }
-    return result;
+    FAIL_IF_NULL(s);
+    DetectEngineCtxFree(de_ctx);
+    PASS;
 }
 
-#endif /* UNITTESTS */
-
-void ReferenceRegisterTests(void)
+static void ReferenceRegisterTests(void)
 {
-#ifdef UNITTESTS
     UtRegisterTest("DetectReferenceParseTest01", DetectReferenceParseTest01);
     UtRegisterTest("DetectReferenceParseTest02", DetectReferenceParseTest02);
     UtRegisterTest("DetectReferenceParseTest03", DetectReferenceParseTest03);
-#endif /* UNITTESTS */
-
-    return;
 }
+#endif /* UNITTESTS */
