@@ -21,23 +21,18 @@
  * \author Eric Leblond <eric@regit.org>
  */
 
-#ifndef __DETECT_REPLACE_H__
-#define __DETECT_REPLACE_H__
+#ifndef SURICATA_DETECT_REPLACE_H
+#define SURICATA_DETECT_REPLACE_H
 
-DetectReplaceList * DetectReplaceAddToList(DetectReplaceList *replist, uint8_t *found, DetectContentData *cd);
+#include "detect-content.h"
+
+DetectReplaceList *DetectReplaceAddToList(
+        DetectReplaceList *replist, uint8_t *found, const DetectContentData *cd);
 
 /* Internal functions are only called via the inline functions below. */
 void DetectReplaceExecuteInternal(Packet *p, DetectReplaceList *replist);
 void DetectReplaceFreeInternal(DetectReplaceList *replist);
 
-static inline void DetectReplaceExecute(Packet *p, DetectEngineThreadCtx *det_ctx)
-{
-    if (p == NULL || det_ctx->replist == NULL)
-        return;
-    DetectReplaceExecuteInternal(p, det_ctx->replist);
-    det_ctx->replist = NULL;
-}
-
 static inline void DetectReplaceFree(DetectEngineThreadCtx *det_ctx)
 {
     if (det_ctx->replist) {
