@@ -21,8 +21,10 @@
  * \author Eric Leblond <eleblond@stamus-networks.com>
  */
 
-#ifndef __FLOW_BYPASS_H__
-#define __FLOW_BYPASS_H__
+#ifndef SURICATA_FLOW_BYPASS_H
+#define SURICATA_FLOW_BYPASS_H
+
+#include "flow.h"
 
 struct flows_stats {
     uint64_t count;
@@ -30,17 +32,19 @@ struct flows_stats {
     uint64_t bytes;
 };
 
-typedef int (*BypassedCheckFunc)(struct flows_stats *bypassstats,
-                                 struct timespec *curtime);
-typedef int (*BypassedUpdateFunc)(Flow *f, Packet *p);
-
-void FlowAddToBypassed(Flow *f);
+typedef int (*BypassedCheckFunc)(ThreadVars *th_v,
+                                 struct flows_stats *bypassstats,
+                                 struct timespec *curtime, void *data);
+typedef int (*BypassedCheckFuncInit)(ThreadVars *th_v,
+                                     struct timespec *curtime, void *data);
+typedef int (*BypassedUpdateFunc)(Flow *f, Packet *p, void *data);
 
 void BypassedFlowManagerThreadSpawn(void);
 void TmModuleBypassedFlowManagerRegister(void);
 
-int BypassedFlowManagerRegisterCheckFunc(BypassedCheckFunc CheckFunc);
-int BypassedFlowManagerRegisterUpdateFunc(BypassedUpdateFunc UpdateFunc);
+int BypassedFlowManagerRegisterCheckFunc(BypassedCheckFunc CheckFunc,
+                                         BypassedCheckFuncInit CheckFuncInit, void *data);
+int BypassedFlowManagerRegisterUpdateFunc(BypassedUpdateFunc UpdateFunc, void *data);
 
 void BypassedFlowUpdate(Flow *f, Packet *p);
 
