@@ -21,8 +21,10 @@
  *  \author Victor Julien <victor@inliniac.net>
  */
 
-#ifndef __FLOW_HASH_H__
-#define __FLOW_HASH_H__
+#ifndef SURICATA_FLOW_HASH_H
+#define SURICATA_FLOW_HASH_H
+
+#include "flow.h"
 
 /** Spinlocks or Mutex for the flow buckets. */
 //#define FBLOCK_SPIN
@@ -39,8 +41,11 @@
  * the same hashkey (the hash is a chained hash). When doing modifications
  * to the list, the entire bucket is locked. */
 typedef struct FlowBucket_ {
+    /** head of the list of active flows for this row. */
     Flow *head;
-    Flow *tail;
+    /** head of the list of evicted flows for this row. Waiting to be
+     *  collected by the Flow Manager. */
+    Flow *evicted;
 #ifdef FBLOCK_MUTEX
     SCMutex m;
 #elif defined FBLOCK_SPIN
@@ -51,9 +56,9 @@ typedef struct FlowBucket_ {
     /** timestamp in seconds of the earliest possible moment a flow
      *  will time out in this row. Set by the flow manager. Cleared
      *  to 0 by workers, either when new flows are added or when a
-     *  flow state changes. The flow manager sets this to INT_MAX for
+     *  flow state changes. The flow manager sets this to UINT_MAX for
      *  empty buckets. */
-    SC_ATOMIC_DECLARE(int32_t, next_ts);
+    SC_ATOMIC_DECLARE(uint32_t, next_ts);
 } __attribute__((aligned(CLS))) FlowBucket;
 
 #ifdef FBLOCK_SPIN
@@ -74,9 +79,27 @@ typedef struct FlowBucket_ {
 
 /* prototypes */
 
-Flow *FlowGetFlowFromHash(ThreadVars *tv, DecodeThreadVars *dtv, const Packet *, Flow **);
+Flow *FlowGetFlowFromHash(ThreadVars *tv, FlowLookupStruct *tctx, Packet *, Flow **);
 
-void FlowDisableTcpReuseHandling(void);
+Flow *FlowGetFromFlowKey(FlowKey *key, struct timespec *ttime, const uint32_t hash);
+Flow *FlowGetExistingFlowFromFlowId(uint64_t flow_id);
+uint32_t FlowKeyGetHash(FlowKey *flow_key);
+uint32_t FlowGetIpPairProtoHash(const Packet *p);
 
-#endif /* __FLOW_HASH_H__ */
+/** \note f->fb must be locked */
+static inline void RemoveFromHash(Flow *f, Flow *prev_f)
+{
+    FlowBucket *fb = f->fb;
 
+    /* remove from the hash */
+    if (prev_f != NULL) {
+        prev_f->next = f->next;
+    } else {
+        fb->head = f->next;
+    }
+
+    f->next = NULL;
+    f->fb = NULL;
+}
+
+#endif /* SURICATA_FLOW_HASH_H */
