@@ -21,11 +21,10 @@
  * \author Victor Julien <victor@inliniac.net>
  */
 
-#ifndef __FLOW_QUEUE_H__
-#define __FLOW_QUEUE_H__
+#ifndef SURICATA_FLOW_QUEUE_H
+#define SURICATA_FLOW_QUEUE_H
 
 #include "suricata-common.h"
-#include "flow.h"
 
 /** Spinlocks or Mutex for the flow queues. */
 //#define FQLOCK_SPIN
@@ -37,15 +36,18 @@
     #endif
 #endif
 
-/* Define a queue for storing flows */
-typedef struct FlowQueue_
+typedef struct FlowQueuePrivate_
 {
     Flow *top;
     Flow *bot;
     uint32_t len;
-#ifdef DBG_PERF
-    uint32_t dbg_maxlen;
-#endif /* DBG_PERF */
+} FlowQueuePrivate;
+
+/* Define a queue for storing flows */
+typedef struct FlowQueue_
+{
+    FlowQueuePrivate priv;
+    SC_ATOMIC_DECLARE(bool,non_empty);
 #ifdef FQLOCK_MUTEX
     SCMutex m;
 #elif defined FQLOCK_SPIN
@@ -54,6 +56,9 @@ typedef struct FlowQueue_
     #error Enable FQLOCK_SPIN or FQLOCK_MUTEX
 #endif
 } FlowQueue;
+#define qtop priv.top
+#define qbot priv.bot
+#define qlen priv.len
 
 #ifdef FQLOCK_SPIN
     #define FQLOCK_INIT(q) SCSpinInit(&(q)->s, 0)
@@ -78,8 +83,14 @@ void FlowQueueDestroy (FlowQueue *);
 
 void FlowEnqueue (FlowQueue *, Flow *);
 Flow *FlowDequeue (FlowQueue *);
+void FlowQueueRemove(FlowQueue *fq, Flow *f);
 
-void FlowMoveToSpare(Flow *);
+void FlowQueuePrivateAppendFlow(FlowQueuePrivate *fqc, Flow *f);
+void FlowQueuePrivatePrependFlow(FlowQueuePrivate *fqc, Flow *f);
 
-#endif /* __FLOW_QUEUE_H__ */
+void FlowQueueAppendPrivate(FlowQueue *fq, FlowQueuePrivate *fqp);
+void FlowQueuePrivateAppendPrivate(FlowQueuePrivate *dest, FlowQueuePrivate *src);
+FlowQueuePrivate FlowQueueExtractPrivate(FlowQueue *fq);
+Flow *FlowQueuePrivateGetFromTop(FlowQueuePrivate *fqp);
 
+#endif /* SURICATA_FLOW_QUEUE_H */
