@@ -21,9 +21,9 @@
  * \author Victor Julien <victor@inliniac.net>
  */
 
-#ifndef __LOG_STATS_H__
-#define __LOG_STATS_H__
+#ifndef SURICATA_LOG_STATS_H
+#define SURICATA_LOG_STATS_H
 
-void TmModuleLogStatsLogRegister (void);
+void LogStatsLogRegister(void);
 
 #endif /* __LOG_HTTPLOG_H__ */
