@@ -15,16 +15,12 @@
  * 02110-1301, USA.
  */
 
-#ifndef __OUTPUT_JSON_DNP3_H__
-#define __OUTPUT_JSON_DNP3_H__
+#ifndef SURICATA_OUTPUT_JSON_DNP3_H
+#define SURICATA_OUTPUT_JSON_DNP3_H
 
 #include "app-layer-dnp3.h"
 
-#ifdef HAVE_LIBJANSSON
-json_t *JsonDNP3LogRequest(DNP3Transaction *);
-json_t *JsonDNP3LogResponse(DNP3Transaction *);
-#endif /* HAVE_LIBJANSSON */
-
 void JsonDNP3LogRegister(void);
+bool AlertJsonDnp3(void *vtx, SCJsonBuilder *js);
 
-#endif /* __OUTPUT_JSON_DNP3_H__ */
+#endif /* SURICATA_OUTPUT_JSON_DNP3_H */
