@@ -22,9 +22,9 @@
  *
  */
 
-#ifndef __OUTPUT_JSON_DROP_H__
-#define __OUTPUT_JSON_DROP_H__
+#ifndef SURICATA_OUTPUT_JSON_DROP_H
+#define SURICATA_OUTPUT_JSON_DROP_H
 
-void TmModuleJsonDropLogRegister (void);
+void JsonDropLogRegister(void);
 
-#endif /* __OUTPUT_DROPLOG_H__ */
+#endif /* SURICATA_OUTPUT_JSON_DROP_H */
