@@ -1,4 +1,4 @@
-/* Copyright (C) 2015-2018 Open Information Security Foundation
+/* Copyright (C) 2015-2021 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -24,7 +24,6 @@
  */
 
 #include "suricata-common.h"
-#include "debug.h"
 #include "detect.h"
 #include "pkt-var.h"
 #include "conf.h"
@@ -46,35 +45,30 @@
 
 #include "output-json-nfs.h"
 
-#ifdef HAVE_RUST
-#ifdef HAVE_LIBJANSSON
 #include "rust.h"
-#include "rust-nfs-log-gen.h"
 
-json_t *JsonNFSAddMetadataRPC(const Flow *f, uint64_t tx_id)
+bool EveNFSAddMetadataRPC(const Flow *f, uint64_t tx_id, SCJsonBuilder *jb)
 {
     NFSState *state = FlowGetAppState(f);
     if (state) {
         NFSTransaction *tx = AppLayerParserGetTx(f->proto, ALPROTO_NFS, state, tx_id);
         if (tx) {
-            return rs_rpc_log_json_response(tx);
+            return SCNfsRpcLogJsonResponse(tx, jb);
         }
     }
-
-    return NULL;
+    return false;
 }
 
-json_t *JsonNFSAddMetadata(const Flow *f, uint64_t tx_id)
+bool EveNFSAddMetadata(const Flow *f, uint64_t tx_id, SCJsonBuilder *jb)
 {
     NFSState *state = FlowGetAppState(f);
     if (state) {
         NFSTransaction *tx = AppLayerParserGetTx(f->proto, ALPROTO_NFS, state, tx_id);
         if (tx) {
-            return rs_nfs_log_json_response(state, tx);
+            return SCNfsLogJsonResponse(state, tx, jb);
         }
     }
-
-    return NULL;
+    return false;
 }
 
 static int JsonNFSLogger(ThreadVars *tv, void *thread_data,
@@ -83,70 +77,40 @@ static int JsonNFSLogger(ThreadVars *tv, void *thread_data,
     NFSTransaction *nfstx = tx;
     OutputJsonThreadCtx *thread = thread_data;
 
-    if (rs_nfs_tx_logging_is_filtered(state, nfstx))
+    if (SCNfsTxLoggingIsFiltered(state, nfstx))
         return TM_ECODE_OK;
 
-    json_t *js = CreateJSONHeader(p, LOG_DIR_PACKET, "nfs");
-    if (unlikely(js == NULL)) {
-        return TM_ECODE_FAILED;
+    SCJsonBuilder *jb = CreateEveHeader(p, LOG_DIR_PACKET, "nfs", NULL, thread->ctx);
+    if (unlikely(jb == NULL)) {
+        return TM_ECODE_OK;
     }
 
-    JsonAddCommonOptions(&thread->ctx->cfg, p, f, js);
+    SCJbOpenObject(jb, "rpc");
+    SCNfsRpcLogJsonResponse(tx, jb);
+    SCJbClose(jb);
 
-    json_t *rpcjs = rs_rpc_log_json_response(tx);
-    if (unlikely(rpcjs == NULL)) {
-        goto error;
-    }
-    json_object_set_new(js, "rpc", rpcjs);
-
-    json_t *nfsjs = rs_nfs_log_json_response(state, tx);
-    if (unlikely(nfsjs == NULL)) {
-        goto error;
-    }
-    json_object_set_new(js, "nfs", nfsjs);
+    SCJbOpenObject(jb, "nfs");
+    SCNfsLogJsonResponse(state, tx, jb);
+    SCJbClose(jb);
 
     MemBufferReset(thread->buffer);
-    OutputJSONBuffer(js, thread->ctx->file_ctx, &thread->buffer);
-
-    json_decref(js);
+    OutputJsonBuilderBuffer(tv, p, p->flow, jb, thread);
+    SCJbFree(jb);
     return TM_ECODE_OK;
-
-error:
-    json_decref(js);
-    return TM_ECODE_FAILED;
 }
 
-static OutputInitResult NFSLogInitSub(ConfNode *conf,
-    OutputCtx *parent_ctx)
+static OutputInitResult NFSLogInitSub(SCConfNode *conf, OutputCtx *parent_ctx)
 {
-    AppLayerParserRegisterLogger(IPPROTO_TCP, ALPROTO_NFS);
-    AppLayerParserRegisterLogger(IPPROTO_UDP, ALPROTO_NFS);
+    SCAppLayerParserRegisterLogger(IPPROTO_TCP, ALPROTO_NFS);
+    SCAppLayerParserRegisterLogger(IPPROTO_UDP, ALPROTO_NFS);
     return OutputJsonLogInitSub(conf, parent_ctx);
 }
 
 void JsonNFSLogRegister(void)
 {
     /* Register as an eve sub-module. */
-    OutputRegisterTxSubModule(LOGGER_JSON_NFS, "eve-log", "JsonNFSLog",
-        "eve-log.nfs", NFSLogInitSub, ALPROTO_NFS,
-        JsonNFSLogger, JsonLogThreadInit,
-        JsonLogThreadDeinit, NULL);
+    OutputRegisterTxSubModule(LOGGER_JSON_TX, "eve-log", "JsonNFSLog", "eve-log.nfs", NFSLogInitSub,
+            ALPROTO_NFS, JsonNFSLogger, JsonLogThreadInit, JsonLogThreadDeinit);
 
     SCLogDebug("NFS JSON logger registered.");
 }
-
-#else /* No JSON support. */
-
-void JsonNFSLogRegister(void)
-{
-}
-
-#endif /* HAVE_LIBJANSSON */
-
-#else /* no rust */
-
-void JsonNFSLogRegister(void)
-{
-}
-
-#endif /* HAVE_RUST */
