@@ -21,11 +21,10 @@
  * \author Victor Julien <victor@inliniac.net>
  */
 
-#ifndef __OUTPUT_JSON_SMB_H__
-#define __OUTPUT_JSON_SMB_H__
+#ifndef SURICATA_OUTPUT_JSON_SMB_H
+#define SURICATA_OUTPUT_JSON_SMB_H
 
 void JsonSMBLogRegister(void);
-#ifdef HAVE_RUST
-json_t *JsonSMBAddMetadata(const Flow *f, uint64_t tx_id);
-#endif /* HAVE_RUST */
-#endif /* __OUTPUT_JSON_SMB_H__ */
+bool EveSMBAddMetadata(const Flow *f, uint64_t tx_id, SCJsonBuilder *jb);
+
+#endif /* SURICATA_OUTPUT_JSON_SMB_H */
