@@ -21,12 +21,10 @@
  * \author Tom DeCanio <td@npulsetech.com>
  */
 
-#ifndef __OUTPUT_JSON_SMTP_H__
-#define __OUTPUT_JSON_SMTP_H__
+#ifndef SURICATA_OUTPUT_JSON_SMTP_H
+#define SURICATA_OUTPUT_JSON_SMTP_H
 
 void JsonSmtpLogRegister(void);
-#ifdef HAVE_LIBJANSSON
-json_t *JsonSMTPAddMetadata(const Flow *f, uint64_t tx_id);
-#endif
+bool EveSMTPAddMetadata(const Flow *f, uint64_t tx_id, SCJsonBuilder *js);
 
-#endif /* __OUTPUT_JSON_SMTP_H__ */
+#endif /* SURICATA_OUTPUT_JSON_SMTP_H */
