@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2012 Open Information Security Foundation
+/* Copyright (C) 2007-2024 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -21,16 +21,14 @@
  * \author Tom DeCanio <td@npulsetech.com>
  */
 
-#ifndef __OUTPUT_JSON_TLS_H__
-#define __OUTPUT_JSON_TLS_H__
+#ifndef SURICATA_OUTPUT_JSON_TLS_H
+#define SURICATA_OUTPUT_JSON_TLS_H
 
 void JsonTlsLogRegister(void);
 
-#ifdef HAVE_LIBJANSSON
-#include "app-layer-ssl.h"
+/* For SCJsonBuilder. */
+#include "rust.h"
 
-void JsonTlsLogJSONBasic(json_t *js, SSLState *ssl_state);
-void JsonTlsLogJSONExtended(json_t *js, SSLState *ssl_state);
-#endif /* HAVE_LIBJANSSON */
+bool JsonTlsLogJSONExtended(void *vtx, SCJsonBuilder *js);
 
-#endif /* __OUTPUT_JSON_TLS_H__ */
+#endif /* SURICATA_OUTPUT_JSON_TLS_H */
