@@ -39,28 +39,20 @@
 #include "util-affinity.h"
 #include "util-runmodes.h"
 #include "source-ipfw.h"
-#include "util-device.h"
-
-static const char *default_mode;
+#include "util-device-private.h"
 
 const char *RunModeIpsIPFWGetDefaultMode(void)
 {
-    return default_mode;
+    return "autofp";
 }
 
 void RunModeIpsIPFWRegister(void)
 {
-    default_mode = "autofp";
-
     RunModeRegisterNewRunMode(RUNMODE_IPFW, "autofp",
-                              "Multi threaded IPFW IPS mode with respect to flow",
-                              RunModeIpsIPFWAutoFp);
+            "Multi threaded IPFW IPS mode with respect to flow", RunModeIpsIPFWAutoFp, NULL);
 
     RunModeRegisterNewRunMode(RUNMODE_IPFW, "workers",
-                              "Multi queue IPFW IPS mode with one thread per queue",
-                              RunModeIpsIPFWWorker);
-
-    return;
+            "Multi queue IPFW IPS mode with one thread per queue", RunModeIpsIPFWWorker, NULL);
 }
 
 int RunModeIpsIPFWAutoFp(void)
@@ -69,8 +61,6 @@ int RunModeIpsIPFWAutoFp(void)
     int ret = 0;
 #ifdef IPFW
 
-    RunModeInitialize();
-
     TimeModeSetLive();
 
     LiveDeviceHasNoStats();
@@ -89,8 +79,6 @@ int RunModeIpsIPFWWorker(void)
     int ret = 0;
 #ifdef IPFW
 
-    RunModeInitialize();
-
     TimeModeSetLive();
 
     LiveDeviceHasNoStats();
