@@ -21,15 +21,14 @@
  * \author Victor Julien <victor@inliniac.net>
  */
 
-#ifndef __SOURCE_PCAP_FILE_H__
-#define __SOURCE_PCAP_FILE_H__
+#ifndef SURICATA_SOURCE_PCAP_FILE_H
+#define SURICATA_SOURCE_PCAP_FILE_H
 
 void TmModuleReceivePcapFileRegister (void);
 void TmModuleDecodePcapFileRegister (void);
 
-void PcapIncreaseInvalidChecksum();
+void PcapIncreaseInvalidChecksum(void);
 
-void PcapFileGlobalInit();
-
-#endif /* __SOURCE_PCAP_FILE_H__ */
+void PcapFileGlobalInit(void);
 
+#endif /* SURICATA_SOURCE_PCAP_FILE_H */
