@@ -21,23 +21,24 @@
  * \author Victor Julien <victor@inliniac.net>
  */
 
-#ifndef __STREAM_H__
-#define __STREAM_H__
+#ifndef SURICATA_STREAM_H
+#define SURICATA_STREAM_H
 
-#include "flow.h"
+#include "decode.h"
+#include "stream-tcp-private.h"
 
-#define STREAM_START            0x01
-#define STREAM_EOF              0x02
-#define STREAM_TOSERVER         0x04
-#define STREAM_TOCLIENT         0x08
-#define STREAM_GAP              0x10    /**< data gap encountered */
-#define STREAM_DEPTH            0x20    /**< depth reached */
-#define STREAM_FLUSH            0x80
+#define STREAM_FLAGS_FOR_PACKET(p) (PKT_IS_TOSERVER((p)) ? STREAM_TOSERVER : STREAM_TOCLIENT)
 
-typedef int (*StreamSegmentCallback)(const Packet *, void *, const uint8_t *, uint32_t);
+#define STREAM_DUMP_TOCLIENT BIT_U8(1)
+#define STREAM_DUMP_TOSERVER BIT_U8(2)
+#define STREAM_DUMP_HEADERS  BIT_U8(3)
+
+typedef int (*StreamSegmentCallback)(
+        const Packet *, TcpSegment *, void *, const uint8_t *, uint32_t);
 int StreamSegmentForEach(const Packet *p, uint8_t flag,
                          StreamSegmentCallback CallbackFunc,
                          void *data);
+int StreamSegmentForSession(
+        const Packet *p, uint8_t flag, StreamSegmentCallback CallbackFunc, void *data);
 
-#endif /* __STREAM_H__ */
-
+#endif /* SURICATA_STREAM_H */
