@@ -23,11 +23,23 @@
  * Lock profiling wrappers
  */
 
-#ifndef __THREADS_PROFILE_H__
-#define __THREADS_PROFILE_H__
+#ifndef SURICATA_THREADS_PROFILE_H
+#define SURICATA_THREADS_PROFILE_H
+
+// UtilCpuGetTicks
+#include "util-cpu.h"
+
+#define PROFILING_MAX_LOCKS 64
 
 /* profiling */
 
+enum {
+    LOCK_MUTEX,
+    LOCK_SPIN,
+    LOCK_RWW, /**< rwlock, writer */
+    LOCK_RWR, /**< rwlock, reader */
+};
+
 typedef struct ProfilingLock_ {
     char *file;
     char *func;
@@ -37,13 +49,13 @@ typedef struct ProfilingLock_ {
     uint64_t ticks;
 } ProfilingLock;
 
-extern __thread ProfilingLock locks[PROFILING_MAX_LOCKS];
-extern __thread int locks_idx;
-extern __thread int record_locks;
+extern thread_local ProfilingLock locks[PROFILING_MAX_LOCKS];
+extern thread_local int locks_idx;
+extern thread_local int record_locks;
 
-extern __thread uint64_t mutex_lock_contention;
-extern __thread uint64_t mutex_lock_wait_ticks;
-extern __thread uint64_t mutex_lock_cnt;
+extern thread_local uint64_t mutex_lock_contention;
+extern thread_local uint64_t mutex_lock_wait_ticks;
+extern thread_local uint64_t mutex_lock_cnt;
 
 /* mutex */
 
@@ -92,9 +104,9 @@ extern __thread uint64_t mutex_lock_cnt;
 
 /* spinlocks */
 
-extern __thread uint64_t spin_lock_contention;
-extern __thread uint64_t spin_lock_wait_ticks;
-extern __thread uint64_t spin_lock_cnt;
+extern thread_local uint64_t spin_lock_contention;
+extern thread_local uint64_t spin_lock_wait_ticks;
+extern thread_local uint64_t spin_lock_cnt;
 
 //printf("%16s(%s:%d): (thread:%"PRIuMAX") locked mutex %p ret %" PRId32 "\n", __FUNCTION__, __FILE__, __LINE__, (uintmax_t)pthread_self(), mut, retl);
 #define SCSpinLock_profile(spin) ({ \
@@ -131,9 +143,9 @@ extern __thread uint64_t spin_lock_cnt;
 
 /* rwlocks */
 
-extern __thread uint64_t rww_lock_contention;
-extern __thread uint64_t rww_lock_wait_ticks;
-extern __thread uint64_t rww_lock_cnt;
+extern thread_local uint64_t rww_lock_contention;
+extern thread_local uint64_t rww_lock_wait_ticks;
+extern thread_local uint64_t rww_lock_cnt;
 
 #define SCRWLockWRLock_profile(mut) ({ \
     rww_lock_cnt++; \
@@ -160,9 +172,9 @@ extern __thread uint64_t rww_lock_cnt;
     retl; \
 })
 
-extern __thread uint64_t rwr_lock_contention;
-extern __thread uint64_t rwr_lock_wait_ticks;
-extern __thread uint64_t rwr_lock_cnt;
+extern thread_local uint64_t rwr_lock_contention;
+extern thread_local uint64_t rwr_lock_wait_ticks;
+extern thread_local uint64_t rwr_lock_cnt;
 
 #define SCRWLockRDLock_profile(mut) ({ \
     rwr_lock_cnt++; \
