@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2013 Open Information Security Foundation
+/* Copyright (C) 2007-2020 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -23,7 +23,7 @@
  *
  * Simple output queue handler that makes sure all packets of the same flow
  * are sent to the same queue. We support different kind of q handlers.  Have
- * a look at "autofp-scheduler" conf to further undertsand the various q
+ * a look at "autofp-scheduler" conf to further understand the various q
  * handlers we provide.
  */
 
@@ -33,6 +33,7 @@
 #include "threads.h"
 #include "threadvars.h"
 #include "tmqh-flow.h"
+#include "flow-hash.h"
 
 #include "tm-queuehandlers.h"
 
@@ -42,6 +43,7 @@
 Packet *TmqhInputFlow(ThreadVars *t);
 void TmqhOutputFlowHash(ThreadVars *t, Packet *p);
 void TmqhOutputFlowIPPair(ThreadVars *t, Packet *p);
+static void TmqhOutputFlowFTPHash(ThreadVars *t, Packet *p);
 void *TmqhOutputFlowSetupCtx(const char *queue_str);
 void TmqhOutputFlowFreeCtx(void *ctx);
 void TmqhFlowRegisterTests(void);
@@ -55,7 +57,7 @@ void TmqhFlowRegister(void)
     tmqh_table[TMQH_FLOW].RegisterTests = TmqhFlowRegisterTests;
 
     const char *scheduler = NULL;
-    if (ConfGet("autofp-scheduler", &scheduler) == 1) {
+    if (SCConfGet("autofp-scheduler", &scheduler) == 1) {
         if (strcasecmp(scheduler, "round-robin") == 0) {
             SCLogNotice("using flow hash instead of round robin");
             tmqh_table[TMQH_FLOW].OutHandler = TmqhOutputFlowHash;
@@ -66,17 +68,17 @@ void TmqhFlowRegister(void)
             tmqh_table[TMQH_FLOW].OutHandler = TmqhOutputFlowHash;
         } else if (strcasecmp(scheduler, "ippair") == 0) {
             tmqh_table[TMQH_FLOW].OutHandler = TmqhOutputFlowIPPair;
+        } else if (strcasecmp(scheduler, "ftp-hash") == 0) {
+            tmqh_table[TMQH_FLOW].OutHandler = TmqhOutputFlowFTPHash;
         } else {
-            SCLogError(SC_ERR_INVALID_YAML_CONF_ENTRY, "Invalid entry \"%s\" "
+            SCLogError("Invalid entry \"%s\" "
                        "for autofp-scheduler in conf.  Killing engine.",
-                       scheduler);
+                    scheduler);
             exit(EXIT_FAILURE);
         }
     } else {
         tmqh_table[TMQH_FLOW].OutHandler = TmqhOutputFlowHash;
     }
-
-    return;
 }
 
 void TmqhFlowPrintAutofpHandler(void)
@@ -87,6 +89,7 @@ void TmqhFlowPrintAutofpHandler(void)
 
     PRINT_IF_FUNC(TmqhOutputFlowHash, "Hash");
     PRINT_IF_FUNC(TmqhOutputFlowIPPair, "IPPair");
+    PRINT_IF_FUNC(TmqhOutputFlowFTPHash, "FTPHash");
 
 #undef PRINT_IF_FUNC
 }
@@ -94,7 +97,7 @@ void TmqhFlowPrintAutofpHandler(void)
 /* same as 'simple' */
 Packet *TmqhInputFlow(ThreadVars *tv)
 {
-    PacketQueue *q = &trans_q[tv->inq->id];
+    PacketQueue *q = tv->inq->pq;
 
     StatsSyncCountersIfSignalled(tv);
 
@@ -126,15 +129,12 @@ static int StoreQueueId(TmqhFlowCtx *ctx, char *name)
     }
     tmq->writer_cnt++;
 
-    uint16_t id = tmq->id;
-
     if (ctx->queues == NULL) {
         ctx->size = 1;
-        ctx->queues = SCMalloc(ctx->size * sizeof(TmqhFlowMode));
+        ctx->queues = SCCalloc(1, ctx->size * sizeof(TmqhFlowMode));
         if (ctx->queues == NULL) {
             return -1;
         }
-        memset(ctx->queues, 0, ctx->size * sizeof(TmqhFlowMode));
     } else {
         ctx->size++;
         ptmp = SCRealloc(ctx->queues, ctx->size * sizeof(TmqhFlowMode));
@@ -147,7 +147,7 @@ static int StoreQueueId(TmqhFlowCtx *ctx, char *name)
 
         memset(ctx->queues + (ctx->size - 1), 0, sizeof(TmqhFlowMode));
     }
-    ctx->queues[ctx->size - 1].q = &trans_q[id];
+    ctx->queues[ctx->size - 1].q = tmq->pq;
 
     return 0;
 }
@@ -169,10 +169,9 @@ void *TmqhOutputFlowSetupCtx(const char *queue_str)
 
     SCLogDebug("queue_str %s", queue_str);
 
-    TmqhFlowCtx *ctx = SCMalloc(sizeof(TmqhFlowCtx));
+    TmqhFlowCtx *ctx = SCCalloc(1, sizeof(TmqhFlowCtx));
     if (unlikely(ctx == NULL))
         return NULL;
-    memset(ctx,0x00,sizeof(TmqhFlowCtx));
 
     char *str = SCStrdup(queue_str);
     if (unlikely(str == NULL)) {
@@ -216,14 +215,11 @@ void TmqhOutputFlowFreeCtx(void *ctx)
               fctx->size);
     SCFree(fctx->queues);
     SCFree(fctx);
-
-    return;
 }
 
 void TmqhOutputFlowHash(ThreadVars *tv, Packet *p)
 {
-    int16_t qid = 0;
-
+    uint32_t qid;
     TmqhFlowCtx *ctx = (TmqhFlowCtx *)tv->outctx;
 
     if (p->flags & PKT_WANTS_FLOW) {
@@ -241,8 +237,6 @@ void TmqhOutputFlowHash(ThreadVars *tv, Packet *p)
     PacketEnqueue(q, p);
     SCCondSignal(&q->cond_q);
     SCMutexUnlock(&q->mutex_q);
-
-    return;
 }
 
 /**
@@ -253,169 +247,148 @@ void TmqhOutputFlowHash(ThreadVars *tv, Packet *p)
  */
 void TmqhOutputFlowIPPair(ThreadVars *tv, Packet *p)
 {
-    int16_t qid = 0;
     uint32_t addr_hash = 0;
-    int i;
 
     TmqhFlowCtx *ctx = (TmqhFlowCtx *)tv->outctx;
 
     if (p->src.family == AF_INET6) {
-        for (i = 0; i < 4; i++) {
+        for (int i = 0; i < 4; i++) {
             addr_hash += p->src.addr_data32[i] + p->dst.addr_data32[i];
         }
     } else {
         addr_hash = p->src.addr_data32[0] + p->dst.addr_data32[0];
     }
 
-    /* we don't have to worry about possible overflow, since
-     * ctx->size will be lesser than 2 ** 31 for sure */
-    qid = addr_hash % ctx->size;
+    uint32_t qid = addr_hash % ctx->size;
+    PacketQueue *q = ctx->queues[qid].q;
+    SCMutexLock(&q->mutex_q);
+    PacketEnqueue(q, p);
+    SCCondSignal(&q->cond_q);
+    SCMutexUnlock(&q->mutex_q);
+}
+
+static void TmqhOutputFlowFTPHash(ThreadVars *tv, Packet *p)
+{
+    uint32_t qid;
+    TmqhFlowCtx *ctx = (TmqhFlowCtx *)tv->outctx;
+
+    if (p->flags & PKT_WANTS_FLOW) {
+        uint32_t hash = p->flow_hash;
+        if (PacketIsTCP(p) && ((p->sp >= 1024 && p->dp >= 1024) || p->dp == 21 || p->sp == 21 ||
+                                      p->dp == 20 || p->sp == 20)) {
+            hash = FlowGetIpPairProtoHash(p);
+        }
+        qid = hash % ctx->size;
+    } else {
+        qid = ctx->last++;
+
+        if (ctx->last == ctx->size)
+            ctx->last = 0;
+    }
 
     PacketQueue *q = ctx->queues[qid].q;
     SCMutexLock(&q->mutex_q);
     PacketEnqueue(q, p);
     SCCondSignal(&q->cond_q);
     SCMutexUnlock(&q->mutex_q);
-
-    return;
 }
 
 #ifdef UNITTESTS
 
 static int TmqhOutputFlowSetupCtxTest01(void)
 {
-    int retval = 0;
-    Tmq *tmq = NULL;
-    TmqhFlowCtx *fctx = NULL;
-
     TmqResetQueues();
 
-    tmq = TmqCreateQueue("queue1");
-    if (tmq == NULL)
-        goto end;
-    tmq = TmqCreateQueue("queue2");
-    if (tmq == NULL)
-        goto end;
-    tmq = TmqCreateQueue("another");
-    if (tmq == NULL)
-        goto end;
-    tmq = TmqCreateQueue("yetanother");
-    if (tmq == NULL)
-        goto end;
+    Tmq *tmq1 = TmqCreateQueue("queue1");
+    FAIL_IF_NULL(tmq1);
+    Tmq *tmq2 = TmqCreateQueue("queue2");
+    FAIL_IF_NULL(tmq2);
+    Tmq *tmq3 = TmqCreateQueue("another");
+    FAIL_IF_NULL(tmq3);
+    Tmq *tmq4 = TmqCreateQueue("yetanother");
+    FAIL_IF_NULL(tmq4);
 
     const char *str = "queue1,queue2,another,yetanother";
     void *ctx = TmqhOutputFlowSetupCtx(str);
+    FAIL_IF_NULL(ctx);
 
-    if (ctx == NULL)
-        goto end;
+    TmqhFlowCtx *fctx = (TmqhFlowCtx *)ctx;
 
-    fctx = (TmqhFlowCtx *)ctx;
+    FAIL_IF_NOT(fctx->size == 4);
 
-    if (fctx->size != 4)
-        goto end;
+    FAIL_IF_NULL(fctx->queues);
 
-    if (fctx->queues == NULL)
-        goto end;
+    FAIL_IF_NOT(fctx->queues[0].q == tmq1->pq);
+    FAIL_IF_NOT(fctx->queues[1].q == tmq2->pq);
+    FAIL_IF_NOT(fctx->queues[2].q == tmq3->pq);
+    FAIL_IF_NOT(fctx->queues[3].q == tmq4->pq);
 
-    if (fctx->queues[0].q != &trans_q[0])
-        goto end;
-    if (fctx->queues[1].q != &trans_q[1])
-        goto end;
-    if (fctx->queues[2].q != &trans_q[2])
-        goto end;
-    if (fctx->queues[3].q != &trans_q[3])
-        goto end;
-
-    retval = 1;
-end:
-    if (fctx != NULL)
-        TmqhOutputFlowFreeCtx(fctx);
+    TmqhOutputFlowFreeCtx(fctx);
     TmqResetQueues();
-    return retval;
+    PASS;
 }
 
 static int TmqhOutputFlowSetupCtxTest02(void)
 {
-    int retval = 0;
-    Tmq *tmq = NULL;
-    TmqhFlowCtx *fctx = NULL;
-
     TmqResetQueues();
 
-    tmq = TmqCreateQueue("queue1");
-    if (tmq == NULL)
-        goto end;
-    tmq = TmqCreateQueue("queue2");
-    if (tmq == NULL)
-        goto end;
-    tmq = TmqCreateQueue("another");
-    if (tmq == NULL)
-        goto end;
-    tmq = TmqCreateQueue("yetanother");
-    if (tmq == NULL)
-        goto end;
+    Tmq *tmq1 = TmqCreateQueue("queue1");
+    FAIL_IF_NULL(tmq1);
+    Tmq *tmq2 = TmqCreateQueue("queue2");
+    FAIL_IF_NULL(tmq2);
+    Tmq *tmq3 = TmqCreateQueue("another");
+    FAIL_IF_NULL(tmq3);
+    Tmq *tmq4 = TmqCreateQueue("yetanother");
+    FAIL_IF_NULL(tmq4);
 
     const char *str = "queue1";
     void *ctx = TmqhOutputFlowSetupCtx(str);
+    FAIL_IF_NULL(ctx);
 
-    if (ctx == NULL)
-        goto end;
+    TmqhFlowCtx *fctx = (TmqhFlowCtx *)ctx;
 
-    fctx = (TmqhFlowCtx *)ctx;
+    FAIL_IF_NOT(fctx->size == 1);
 
-    if (fctx->size != 1)
-        goto end;
+    FAIL_IF_NULL(fctx->queues);
 
-    if (fctx->queues == NULL)
-        goto end;
-
-    if (fctx->queues[0].q != &trans_q[0])
-        goto end;
-
-    retval = 1;
-end:
-    if (fctx != NULL)
-        TmqhOutputFlowFreeCtx(fctx);
+    FAIL_IF_NOT(fctx->queues[0].q == tmq1->pq);
+    TmqhOutputFlowFreeCtx(fctx);
     TmqResetQueues();
-    return retval;
+
+    PASS;
 }
 
 static int TmqhOutputFlowSetupCtxTest03(void)
 {
-    int retval = 0;
-    TmqhFlowCtx *fctx = NULL;
-
     TmqResetQueues();
 
     const char *str = "queue1,queue2,another,yetanother";
     void *ctx = TmqhOutputFlowSetupCtx(str);
+    FAIL_IF_NULL(ctx);
 
-    if (ctx == NULL)
-        goto end;
+    TmqhFlowCtx *fctx = (TmqhFlowCtx *)ctx;
 
-    fctx = (TmqhFlowCtx *)ctx;
+    FAIL_IF_NOT(fctx->size == 4);
 
-    if (fctx->size != 4)
-        goto end;
+    FAIL_IF_NULL(fctx->queues);
 
-    if (fctx->queues == NULL)
-        goto end;
+    Tmq *tmq1 = TmqGetQueueByName("queue1");
+    FAIL_IF_NULL(tmq1);
+    Tmq *tmq2 = TmqGetQueueByName("queue2");
+    FAIL_IF_NULL(tmq2);
+    Tmq *tmq3 = TmqGetQueueByName("another");
+    FAIL_IF_NULL(tmq3);
+    Tmq *tmq4 = TmqGetQueueByName("yetanother");
+    FAIL_IF_NULL(tmq4);
 
-    if (fctx->queues[0].q != &trans_q[0])
-        goto end;
-    if (fctx->queues[1].q != &trans_q[1])
-        goto end;
-    if (fctx->queues[2].q != &trans_q[2])
-        goto end;
-    if (fctx->queues[3].q != &trans_q[3])
-        goto end;
+    FAIL_IF_NOT(fctx->queues[0].q == tmq1->pq);
+    FAIL_IF_NOT(fctx->queues[1].q == tmq2->pq);
+    FAIL_IF_NOT(fctx->queues[2].q == tmq3->pq);
+    FAIL_IF_NOT(fctx->queues[3].q == tmq4->pq);
 
-    retval = 1;
-end:
-    if (fctx != NULL)
-        TmqhOutputFlowFreeCtx(fctx);
+    TmqhOutputFlowFreeCtx(fctx);
     TmqResetQueues();
-    return retval;
+    PASS;
 }
 
 #endif /* UNITTESTS */
@@ -430,6 +403,4 @@ void TmqhFlowRegisterTests(void)
     UtRegisterTest("TmqhOutputFlowSetupCtxTest03",
                    TmqhOutputFlowSetupCtxTest03);
 #endif
-
-    return;
 }
