@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2010 Open Information Security Foundation
+/* Copyright (C) 2007-2022 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -21,18 +21,17 @@
  * \author Anoop Saldanha <anoopsaldanha@gmail.com>
  */
 
-#ifndef __UTIL_HOST_OS_INFO_H__
-#define __UTIL_HOST_OS_INFO_H__
+#ifndef SURICATA_UTIL_HOST_OS_INFO_H
+#define SURICATA_UTIL_HOST_OS_INFO_H
 
 #define SC_HINFO_IS_IPV6 0
 #define SC_HINFO_IS_IPV4 1
 
-int SCHInfoAddHostOSInfo(char *, char *, int);
-int SCHInfoGetHostOSFlavour(char *);
+int SCHInfoAddHostOSInfo(const char *, const char *, int);
 int SCHInfoGetIPv4HostOSFlavour(uint8_t *);
 int SCHInfoGetIPv6HostOSFlavour(uint8_t *);
 void SCHInfoCleanResources(void);
 void SCHInfoLoadFromConfig(void);
 void SCHInfoRegisterTests(void);
 
-#endif /* __UTIL_HOST_OS_INFO_H__ */
+#endif /* SURICATA_UTIL_HOST_OS_INFO_H */
