@@ -26,6 +26,8 @@
 
 #include "suricata-common.h"
 #include "util-ip.h"
+#include "util-byte.h"
+#include "util-debug.h"
 
 /** \brief determine if a string is a valid ipv4 address
  *  \retval bool is addr valid?
@@ -38,8 +40,8 @@ bool IPv4AddressStringIsValid(const char *str)
 
     memset(&addr, 0, sizeof(addr));
 
-    uint32_t len = strlen(str);
-    uint32_t i = 0;
+    size_t len = strlen(str);
+    size_t i = 0;
     for (i = 0; i < len; i++) {
         if (!(str[i] == '.' || isdigit(str[i]))) {
             return false;
@@ -53,7 +55,7 @@ bool IPv4AddressStringIsValid(const char *str)
             dots++;
             alen = 0;
         } else {
-            if (alen >= 4) {
+            if (alen >= 3) {
                 SCLogDebug("too long");
                 return false;
             }
@@ -65,9 +67,9 @@ bool IPv4AddressStringIsValid(const char *str)
 
     addr[dots][alen] = '\0';
     for (int x = 0; x < 4; x++) {
-        int a = atoi(addr[x]);
-        if (a < 0 || a >= 256) {
-            SCLogDebug("out of range");
+        uint8_t a;
+        if (StringParseUint8(&a, 10, 0, (const char *)addr[x]) < 0) {
+            SCLogDebug("invalid value for address byte: %s", addr[x]);
             return false;
         }
     }
@@ -83,8 +85,8 @@ bool IPv6AddressStringIsValid(const char *str)
     int sep = 0;
     bool colon_seen = false;
 
-    uint32_t len = strlen(str);
-    uint32_t i = 0;
+    size_t len = strlen(str);
+    size_t i = 0;
     for (i = 0; i < len && str[i] != 0; i++) {
         if (!(str[i] == '.' || str[i] == ':' ||
             isxdigit(str[i])))
@@ -133,8 +135,7 @@ struct in_addr *ValidateIPV4Address(const char *addr_str)
         return NULL;
 
     if ( (addr = SCMalloc(sizeof(struct in_addr))) == NULL) {
-        SCLogError(SC_ERR_FATAL, "Fatal error encountered in ValidateIPV4Address. Exiting...");
-        exit(EXIT_FAILURE);
+        FatalError("Fatal error encountered in ValidateIPV4Address. Exiting...");
     }
 
     if (inet_pton(AF_INET, addr_str, addr) <= 0) {
@@ -147,7 +148,7 @@ struct in_addr *ValidateIPV4Address(const char *addr_str)
 
 /**
  * \brief Validates an IPV6 address and returns the network endian arranged
- *        version of the IPV6 addresss
+ *        version of the IPV6 address
  *
  * \param addr Pointer to a character string containing an IPV6 address
  *
@@ -163,8 +164,7 @@ struct in6_addr *ValidateIPV6Address(const char *addr_str)
         return NULL;
 
     if ( (addr = SCMalloc(sizeof(struct in6_addr))) == NULL) {
-        SCLogError(SC_ERR_FATAL, "Fatal error encountered in ValidateIPV6Address. Exiting...");
-        exit(EXIT_FAILURE);
+        FatalError("Fatal error encountered in ValidateIPV6Address. Exiting...");
     }
 
     if (inet_pton(AF_INET6, addr_str, addr) <= 0) {
@@ -200,6 +200,4 @@ void MaskIPNetblock(uint8_t *stream, int netmask, int key_bitlen)
         }
         stream[i] &= mask;
     }
-
-    return;
 }
