@@ -1,5 +1,4 @@
-/* vi: set et ts=4: */
-/* Copyright (C) 2007-2016 Open Information Security Foundation
+/* Copyright (C) 2007-2024 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -26,6 +25,8 @@
 #include "suricata-common.h" /* errno.h, string.h, etc. */
 #include "util-log-redis.h"
 #include "util-logopenfile.h"
+#include "util-byte.h"
+#include "util-debug.h"
 
 #ifdef HAVE_LIBHIREDIS
 
@@ -33,33 +34,38 @@
 #include <event2/thread.h>
 #endif /* HAVE_LIBEVENT_PTHREADS */
 
-static const char * redis_lpush_cmd = "LPUSH";
-static const char * redis_rpush_cmd = "RPUSH";
-static const char * redis_publish_cmd = "PUBLISH";
-static const char * redis_default_key = "suricata";
-static const char * redis_default_server = "127.0.0.1";
+static const char *redis_lpush_cmd = "LPUSH";
+static const char *redis_rpush_cmd = "RPUSH";
+static const char *redis_publish_cmd = "PUBLISH";
+static const char *redis_xadd_cmd = "XADD";
+static const char *redis_default_key = "suricata";
+static const char *redis_default_server = "127.0.0.1";
+static const char *redis_default_format = "%s %s %s";
+static const char *redis_stream_format = "%s %s * eve %s";
+static const char *redis_stream_format_maxlen_tmpl = "%s %s MAXLEN %c %d * eve %s";
 
 static int SCConfLogReopenSyncRedis(LogFileCtx *log_ctx);
 static void SCLogFileCloseRedis(LogFileCtx *log_ctx);
 
+#define REDIS_MAX_STREAM_LENGTH_DEFAULT 100000
+
 /**
  * \brief SCLogRedisInit() - Initializes global stuff before threads
  */
-void SCLogRedisInit()
+void SCLogRedisInit(void)
 {
 #ifdef HAVE_LIBEVENT_PTHREADS
     evthread_use_pthreads();
 #endif /* HAVE_LIBEVENT_PTHREADS */
 }
 
-/** \brief SCLogRedisContextAlloc() - Allocates and initalizes redis context
+/** \brief SCLogRedisContextAlloc() - Allocates and initializes redis context
  */
 static SCLogRedisContext *SCLogRedisContextAlloc(void)
 {
     SCLogRedisContext* ctx = (SCLogRedisContext*) SCCalloc(1, sizeof(SCLogRedisContext));
     if (ctx == NULL) {
-        SCLogError(SC_ERR_MEM_ALLOC, "Unable to allocate redis context");
-        exit(EXIT_FAILURE);
+        FatalError("Unable to allocate redis context");
     }
     ctx->sync = NULL;
 #if HAVE_LIBEVENT
@@ -67,6 +73,7 @@ static SCLogRedisContext *SCLogRedisContextAlloc(void)
     ctx->async   = NULL;
 #endif
     ctx->batch_count = 0;
+    ctx->last_push = 0;
     ctx->tried = 0;
 
     return ctx;
@@ -77,14 +84,13 @@ static SCLogRedisContext *SCLogRedisContextAlloc(void)
 static int SCConfLogReopenAsyncRedis(LogFileCtx *log_ctx);
 #include <hiredis/adapters/libevent.h>
 
-/** \brief SCLogRedisAsyncContextAlloc() - Allocates and initalizes redis context with async
+/** \brief SCLogRedisAsyncContextAlloc() - Allocates and initializes redis context with async
  */
 static SCLogRedisContext *SCLogRedisContextAsyncAlloc(void)
 {
     SCLogRedisContext* ctx = (SCLogRedisContext*) SCCalloc(1, sizeof(SCLogRedisContext));
     if (unlikely(ctx == NULL)) {
-        SCLogError(SC_ERR_MEM_ALLOC, "Unable to allocate redis context");
-        exit(EXIT_FAILURE);
+        FatalError("Unable to allocate redis context");
     }
 
     ctx->sync = NULL;
@@ -92,6 +98,7 @@ static SCLogRedisContext *SCLogRedisContextAsyncAlloc(void)
     ctx->ev_base = NULL;
     ctx->connected = 0;
     ctx->batch_count = 0;
+    ctx->last_push = 0;
     ctx->tried = 0;
 
     return ctx;
@@ -100,7 +107,7 @@ static SCLogRedisContext *SCLogRedisContextAsyncAlloc(void)
 /** \brief SCRedisAsyncCommandCallback() Callback when reply from redis happens.
  *  \param ac redis async context
  *  \param r redis reply
- *  \param privvata opaque datq with pointer to LogFileCtx
+ *  \param privdata opaque data with pointer to LogFileCtx
  */
 static void SCRedisAsyncCommandCallback(redisAsyncContext *ac, void *r, void *privdata)
 {
@@ -122,7 +129,7 @@ static void SCRedisAsyncCommandCallback(redisAsyncContext *ac, void *r, void *pr
  *         This is used to check if redis is connected.
  *  \param ac redis async context
  *  \param r redis reply
- *  \param privvata opaque datq with pointer to LogFileCtx
+ *  \param privdata opaque data with pointer to LogFileCtx
  */
 static void SCRedisAsyncEchoCommandCallback(redisAsyncContext *ac, void *r, void *privdata)
 {
@@ -138,7 +145,7 @@ static void SCRedisAsyncEchoCommandCallback(redisAsyncContext *ac, void *r, void
     } else {
        ctx->connected = 0;
        if (ctx->tried == 0) {
-          SCLogWarning(SC_ERR_SOCKET, "Failed to connect to Redis... (will keep trying)");
+           SCLogWarning("Failed to connect to Redis... (will keep trying)");
        }
        ctx->tried = time(NULL);
     }
@@ -160,7 +167,7 @@ static void SCLogAsyncRedisSendEcho(SCLogRedisContext * ctx)
  *         This is used to terminate connection with redis.
  *  \param ac redis async context
  *  \param r redis reply
- *  \param privvata opaque datq with pointer to LogFileCtx
+ *  \param privdata opaque data with pointer to LogFileCtx
  */
 static void SCRedisAsyncQuitCommandCallback(redisAsyncContext *ac, void *r, void *privdata)
 {
@@ -201,20 +208,25 @@ static int SCConfLogReopenAsyncRedis(LogFileCtx *log_ctx)
         return -1;
     }
 
-    ctx->async = redisAsyncConnect(redis_server, redis_port);
+    if (strchr(redis_server, '/') == NULL) {
+        ctx->async = redisAsyncConnect(redis_server, redis_port);
+    } else {
+        ctx->async = redisAsyncConnectUnix(redis_server);
+    }
 
     if (ctx->ev_base != NULL) {
         event_base_free(ctx->ev_base);
+        ctx->ev_base = NULL;
     }
 
     if (ctx->async == NULL) {
-        SCLogError(SC_ERR_MEM_ALLOC, "Error allocate redis async.");
+        SCLogError("Error allocate redis async.");
         ctx->tried = time(NULL);
         return -1;
     }
 
     if (ctx->async != NULL && ctx->async->err) {
-        SCLogError(SC_ERR_SOCKET, "Error setting to redis async: [%s].", ctx->async->errstr);
+        SCLogError("Error setting to redis async: [%s].", ctx->async->errstr);
         ctx->tried = time(NULL);
         return -1;
     }
@@ -262,12 +274,8 @@ static int SCLogRedisWriteAsync(LogFileCtx *file_ctx, const char *string, size_t
         return -1;
     }
 
-    redisAsyncCommand(ctx->async,
-            SCRedisAsyncCommandCallback,
-            file_ctx,
-            "%s %s %s",
-            file_ctx->redis_setup.command,
-            file_ctx->redis_setup.key,
+    redisAsyncCommand(ctx->async, SCRedisAsyncCommandCallback, file_ctx,
+            file_ctx->redis_setup.format, file_ctx->redis_setup.command, file_ctx->redis_setup.key,
             string);
 
     event_base_loop(ctx->ev_base, EVLOOP_NONBLOCK);
@@ -295,14 +303,19 @@ static int SCConfLogReopenSyncRedis(LogFileCtx *log_ctx)
     if (ctx->sync != NULL)  {
         redisFree(ctx->sync);
     }
-    ctx->sync = redisConnect(redis_server, redis_port);
+
+    if (strchr(redis_server, '/') == NULL) {
+        ctx->sync = redisConnect(redis_server, redis_port);
+    } else {
+        ctx->sync = redisConnectUnix(redis_server);
+    }
     if (ctx->sync == NULL) {
-        SCLogError(SC_ERR_SOCKET, "Error connecting to redis server.");
+        SCLogError("Error connecting to redis server.");
         ctx->tried = time(NULL);
         return -1;
     }
     if (ctx->sync->err) {
-        SCLogError(SC_ERR_SOCKET, "Error connecting to redis server: [%s].", ctx->sync->errstr);
+        SCLogError("Error connecting to redis server: [%s].", ctx->sync->errstr);
         redisFree(ctx->sync);
         ctx->sync = NULL;
         ctx->tried = time(NULL);
@@ -334,15 +347,16 @@ static int SCLogRedisWriteSync(LogFileCtx *file_ctx, const char *string)
 
     /* synchronous mode */
     if (file_ctx->redis_setup.batch_size) {
-        redisAppendCommand(redis, "%s %s %s",
-                file_ctx->redis_setup.command,
-                file_ctx->redis_setup.key,
-                string);
-        if (ctx->batch_count == file_ctx->redis_setup.batch_size) {
+        redisAppendCommand(redis, file_ctx->redis_setup.format, file_ctx->redis_setup.command,
+                file_ctx->redis_setup.key, string);
+        time_t now = time(NULL);
+        if ((ctx->batch_count == file_ctx->redis_setup.batch_size) || (ctx->last_push < now)) {
             redisReply *reply;
             int i;
+            int batch_size = ctx->batch_count;
             ctx->batch_count = 0;
-            for (i = 0; i <= file_ctx->redis_setup.batch_size; i++) {
+            ctx->last_push = now;
+            for (i = 0; i <= batch_size; i++) {
                 if (redisGetReply(redis, (void **)&reply) == REDIS_OK) {
                     freeReplyObject(reply);
                     ret = 0;
@@ -360,15 +374,18 @@ static int SCLogRedisWriteSync(LogFileCtx *file_ctx, const char *string)
                             redis = ctx->sync;
                             if (redis) {
                                 SCLogInfo("Reconnected to redis server");
+                                redisAppendCommand(redis, file_ctx->redis_setup.format,
+                                        file_ctx->redis_setup.command, file_ctx->redis_setup.key,
+                                        string);
+                                ctx->batch_count++;
+                                return 0;
                             } else {
                                 SCLogInfo("Unable to reconnect to redis server");
                                 return -1;
                             }
                             break;
                         default:
-                            SCLogWarning(SC_ERR_INVALID_VALUE,
-                                    "Unsupported error code %d",
-                                    redis->err);
+                            SCLogWarning("Unsupported error code %d", redis->err);
                             return -1;
                     }
                 }
@@ -377,24 +394,25 @@ static int SCLogRedisWriteSync(LogFileCtx *file_ctx, const char *string)
             ctx->batch_count++;
         }
     } else {
-        redisReply *reply = redisCommand(redis, "%s %s %s",
-                file_ctx->redis_setup.command,
-                file_ctx->redis_setup.key,
-                string);
+        redisReply *reply = redisCommand(redis, file_ctx->redis_setup.format,
+                file_ctx->redis_setup.command, file_ctx->redis_setup.key, string);
         /* We may lose the reply if disconnection happens*/
         if (reply) {
             switch (reply->type) {
                 case REDIS_REPLY_ERROR:
-                    SCLogWarning(SC_ERR_SOCKET, "Redis error: %s", reply->str);
+                    SCLogWarning("Redis error: %s", reply->str);
                     SCConfLogReopenSyncRedis(file_ctx);
                     break;
                 case REDIS_REPLY_INTEGER:
                     SCLogDebug("Redis integer %lld", reply->integer);
                     ret = 0;
                     break;
+                case REDIS_REPLY_STRING:
+                    SCLogDebug("Redis string %s", reply->str);
+                    ret = 0;
+                    break;
                 default:
-                    SCLogError(SC_ERR_INVALID_VALUE,
-                            "Redis default triggered with %d", reply->type);
+                    SCLogError("Redis default triggered with %d", reply->type);
                     SCConfLogReopenSyncRedis(file_ctx);
                     break;
             }
@@ -411,7 +429,7 @@ static int SCLogRedisWriteSync(LogFileCtx *file_ctx, const char *string)
  * \param log_ctx Log file context allocated by caller
  * \param string buffer with data to write
  * \param string_len data length
- * \retval 0 on sucess;
+ * \retval 0 on success;
  * \retval -1 on failure;
  */
 int LogFileWriteRedis(void *lf_ctx, const char *string, size_t string_len)
@@ -439,23 +457,27 @@ int LogFileWriteRedis(void *lf_ctx, const char *string, size_t string_len)
  *  \param log_ctx Log file context allocated by caller
  *  \retval 0 on success
  */
-int SCConfLogOpenRedis(ConfNode *redis_node, void *lf_ctx)
+int SCConfLogOpenRedis(SCConfNode *redis_node, void *lf_ctx)
 {
     LogFileCtx *log_ctx = lf_ctx;
 
+    if (log_ctx->threaded) {
+        FatalError("redis does not support threaded output");
+    }
+
     const char *redis_port = NULL;
     const char *redis_mode = NULL;
 
     int is_async = 0;
 
     if (redis_node) {
-        log_ctx->redis_setup.server = ConfNodeLookupChildValue(redis_node, "server");
-        log_ctx->redis_setup.key =  ConfNodeLookupChildValue(redis_node, "key");
+        log_ctx->redis_setup.server = SCConfNodeLookupChildValue(redis_node, "server");
+        log_ctx->redis_setup.key = SCConfNodeLookupChildValue(redis_node, "key");
 
-        redis_port =  ConfNodeLookupChildValue(redis_node, "port");
-        redis_mode =  ConfNodeLookupChildValue(redis_node, "mode");
+        redis_port = SCConfNodeLookupChildValue(redis_node, "port");
+        redis_mode = SCConfNodeLookupChildValue(redis_node, "mode");
 
-        (void)ConfGetChildValueBool(redis_node, "async", &is_async);
+        (void)SCConfGetChildValueBool(redis_node, "async", &is_async);
     }
     if (!log_ctx->redis_setup.server) {
         log_ctx->redis_setup.server = redis_default_server;
@@ -471,7 +493,7 @@ int SCConfLogOpenRedis(ConfNode *redis_node, void *lf_ctx)
 
 #ifndef HAVE_LIBEVENT
     if (is_async) {
-        SCLogWarning(SC_ERR_NO_REDIS_ASYNC, "async option not available.");
+        SCLogWarning("async option not available.");
     }
     is_async = 0;
 #endif //ifndef HAVE_LIBEVENT
@@ -479,14 +501,14 @@ int SCConfLogOpenRedis(ConfNode *redis_node, void *lf_ctx)
     log_ctx->redis_setup.is_async = is_async;
     log_ctx->redis_setup.batch_size = 0;
     if (redis_node) {
-        ConfNode *pipelining = ConfNodeLookupChild(redis_node, "pipelining");
+        SCConfNode *pipelining = SCConfNodeLookupChild(redis_node, "pipelining");
         if (pipelining) {
             int enabled = 0;
             int ret;
             intmax_t val;
-            ret = ConfGetChildValueBool(pipelining, "enabled", &enabled);
+            ret = SCConfGetChildValueBool(pipelining, "enabled", &enabled);
             if (ret && enabled) {
-                ret = ConfGetChildValueInt(pipelining, "batch-size", &val);
+                ret = SCConfGetChildValueInt(pipelining, "batch-size", &val);
                 if (ret) {
                     log_ctx->redis_setup.batch_size = val;
                 } else {
@@ -498,23 +520,44 @@ int SCConfLogOpenRedis(ConfNode *redis_node, void *lf_ctx)
         log_ctx->redis_setup.batch_size = 0;
     }
 
+    log_ctx->redis_setup.format = redis_default_format;
     if (!strcmp(redis_mode, "list") || !strcmp(redis_mode,"lpush")) {
         log_ctx->redis_setup.command = redis_lpush_cmd;
     } else if(!strcmp(redis_mode, "rpush")){
         log_ctx->redis_setup.command = redis_rpush_cmd;
     } else if(!strcmp(redis_mode,"channel") || !strcmp(redis_mode,"publish")) {
         log_ctx->redis_setup.command = redis_publish_cmd;
+    } else if (!strcmp(redis_mode, "stream") || !strcmp(redis_mode, "xadd")) {
+        int exact;
+        intmax_t maxlen;
+        log_ctx->redis_setup.command = redis_xadd_cmd;
+        log_ctx->redis_setup.format = redis_stream_format;
+        if (SCConfGetChildValueBool(redis_node, "stream-trim-exact", &exact) == 0) {
+            exact = 0;
+        }
+        if (SCConfGetChildValueInt(redis_node, "stream-maxlen", &maxlen) == 0) {
+            maxlen = REDIS_MAX_STREAM_LENGTH_DEFAULT;
+        }
+        if (maxlen > 0) {
+            /* we do not need a lot of space here since we only build another
+            format string, whose length is limited by the length of the
+            maxlen integer formatted as a string */
+            log_ctx->redis_setup.stream_format = SCCalloc(100, sizeof(char));
+            snprintf(log_ctx->redis_setup.stream_format, 100, redis_stream_format_maxlen_tmpl, "%s",
+                    "%s", exact ? '=' : '~', maxlen, "%s");
+            log_ctx->redis_setup.format = log_ctx->redis_setup.stream_format;
+        }
     } else {
-        SCLogError(SC_ERR_REDIS_CONFIG,"Invalid redis mode");
-        exit(EXIT_FAILURE);
+        FatalError("Invalid redis mode: %s", redis_mode);
     }
 
     /* store server params for reconnection */
     if (!log_ctx->redis_setup.server) {
-        SCLogError(SC_ERR_MEM_ALLOC, "Error allocating redis server string");
-        exit(EXIT_FAILURE);
+        FatalError("Error allocating redis server string");
+    }
+    if (StringParseUint16(&log_ctx->redis_setup.port, 10, 0, (const char *)redis_port) < 0) {
+        FatalError("Invalid value for redis port: %s", redis_port);
     }
-    log_ctx->redis_setup.port = atoi(redis_port);
     log_ctx->Close = SCLogFileCloseRedis;
 
 #ifdef HAVE_LIBEVENT
