@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Open Information Security Foundation
+/* Copyright (C) 2014-2022 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -24,7 +24,6 @@
  */
 
 #include "suricata-common.h"
-#include "debug.h"
 #include "detect.h"
 #include "pkt-var.h"
 #include "conf.h"
@@ -48,33 +47,28 @@
 #include "util-logopenfile.h"
 #include "util-time.h"
 
-#ifdef HAVE_LUA
-
-#include <lua.h>
-#include <lualib.h>
-#include <lauxlib.h>
+#include "lua.h"
+#include "lualib.h"
+#include "lauxlib.h"
 
 #include "util-lua.h"
+#include "util-lua-sandbox.h"
 
 lua_State *LuaGetState(void)
 {
     lua_State *s = NULL;
-#ifdef HAVE_LUAJIT
-    s = LuajitGetState();
-#else
     s = luaL_newstate();
-#endif
     return s;
 }
 
 void LuaReturnState(lua_State *s)
 {
     if (s != NULL) {
-#ifdef HAVE_LUAJIT
-        LuajitReturnState(s);
-#else
+        /* clear the stack */
+        while (lua_gettop(s) > 0) {
+            lua_pop(s, 1);
+        }
         lua_close(s);
-#endif
     }
 }
 
@@ -82,6 +76,8 @@ void LuaReturnState(lua_State *s)
 const char lua_ext_key_tv[] = "suricata:lua:tv:ptr";
 /* key for tx pointer */
 const char lua_ext_key_tx[] = "suricata:lua:tx:ptr";
+/* key for tx id */
+const char lua_ext_key_tx_id[] = "suricata:lua:tx_id";
 /* key for p (packet) pointer */
 const char lua_ext_key_p[] = "suricata:lua:pkt:ptr";
 /* key for f (flow) pointer */
@@ -93,8 +89,12 @@ const char lua_ext_key_direction[] = "suricata:lua:direction";
 
 /* key for pa (packet alert) pointer */
 const char lua_ext_key_pa[] = "suricata:lua:pkt:alert:ptr";
+/* key for s (signature) pointer */
+const char lua_ext_key_s[] = "suricata:lua:signature:ptr";
 /* key for file pointer */
 const char lua_ext_key_file[] = "suricata:lua:file:ptr";
+/* key for DetectEngineThreadCtx pointer */
+const char lua_ext_key_det_ctx[] = "suricata:lua:det_ctx:ptr";
 /* key for streaming buffer pointer */
 const char lua_ext_key_streaming_buffer[] = "suricata:lua:streaming_buffer:ptr";
 
@@ -139,11 +139,23 @@ void *LuaStateGetTX(lua_State *luastate)
     return tx;
 }
 
-void LuaStateSetTX(lua_State *luastate, void *txptr)
+/** \brief get tx id from the lua state */
+uint64_t LuaStateGetTxId(lua_State *luastate)
+{
+    lua_pushlightuserdata(luastate, (void *)&lua_ext_key_tx_id);
+    lua_gettable(luastate, LUA_REGISTRYINDEX);
+    uint64_t tx_id = lua_tointeger(luastate, -1);
+    return tx_id;
+}
+void LuaStateSetTX(lua_State *luastate, void *txptr, const uint64_t tx_id)
 {
     lua_pushlightuserdata(luastate, (void *)&lua_ext_key_tx);
     lua_pushlightuserdata(luastate, (void *)txptr);
     lua_settable(luastate, LUA_REGISTRYINDEX);
+
+    lua_pushlightuserdata(luastate, (void *)&lua_ext_key_tx_id);
+    lua_pushinteger(luastate, tx_id);
+    lua_settable(luastate, LUA_REGISTRYINDEX);
 }
 
 Flow *LuaStateGetFlow(lua_State *luastate)
@@ -191,6 +203,22 @@ void LuaStateSetPacketAlert(lua_State *luastate, PacketAlert *pa)
     lua_settable(luastate, LUA_REGISTRYINDEX);
 }
 
+/** \brief get signature pointer from the lua state */
+Signature *LuaStateGetSignature(lua_State *luastate)
+{
+    lua_pushlightuserdata(luastate, (void *)&lua_ext_key_s);
+    lua_gettable(luastate, LUA_REGISTRYINDEX);
+    void *s = lua_touserdata(luastate, -1);
+    return (Signature *)s;
+}
+
+void LuaStateSetSignature(lua_State *luastate, const Signature *s)
+{
+    lua_pushlightuserdata(luastate, (void *)&lua_ext_key_s);
+    lua_pushlightuserdata(luastate, (void *)s);
+    lua_settable(luastate, LUA_REGISTRYINDEX);
+}
+
 /** \brief get file pointer from the lua state */
 File *LuaStateGetFile(lua_State *luastate)
 {
@@ -207,6 +235,22 @@ void LuaStateSetFile(lua_State *luastate, File *file)
     lua_settable(luastate, LUA_REGISTRYINDEX);
 }
 
+/** \brief get DetectEngineThreadCtx pointer from the lua state */
+DetectEngineThreadCtx *LuaStateGetDetCtx(lua_State *luastate)
+{
+    lua_pushlightuserdata(luastate, (void *)&lua_ext_key_det_ctx);
+    lua_gettable(luastate, LUA_REGISTRYINDEX);
+    void *det_ctx = lua_touserdata(luastate, -1);
+    return (DetectEngineThreadCtx *)det_ctx;
+}
+
+void LuaStateSetDetCtx(lua_State *luastate, DetectEngineThreadCtx *det_ctx)
+{
+    lua_pushlightuserdata(luastate, (void *)&lua_ext_key_det_ctx);
+    lua_pushlightuserdata(luastate, (void *)det_ctx);
+    lua_settable(luastate, LUA_REGISTRYINDEX);
+}
+
 LuaStreamingBuffer *LuaStateGetStreamingBuffer(lua_State *luastate)
 {
     lua_pushlightuserdata(luastate, (void *)&lua_ext_key_streaming_buffer);
@@ -298,5 +342,3 @@ int LuaPushInteger(lua_State *luastate, lua_Integer n)
     lua_pushinteger(luastate, n);
     return 1;
 }
-
-#endif /* HAVE_LUA */
