@@ -21,16 +21,11 @@
  * \author Jan Jezek <jjezek@kerio.com>
  */
 
-#ifndef __WIN32_MISC_H__
-#define __WIN32_MISC_H__
+#ifndef SURICATA_WIN32_MISC_H
+#define SURICATA_WIN32_MISC_H
 
 #include <sdkddkver.h>
 
-#define index strchr
-#define rindex strrchr
-
-#define bzero(s, n) memset(s, 0, n)
-
 #ifndef O_NOFOLLOW
 #define O_NOFOLLOW 0
 #endif /* O_NOFOLLOW */
